SystemOrganization addCategory: #NautilusRefactoring!
SystemOrganization addCategory: #'NautilusRefactoring-Refactorings'!
SystemOrganization addCategory: #'NautilusRefactoring-Utilities'!

!NautilusUI methodsFor: '*NautilusRefactoring' stamp: ''!
dispatchRenameClass: aClass
	" Override to set the class rename with RB by default "

	self renameClassWithRefactoringEngine: aClass! !

!NautilusUI methodsFor: '*NautilusRefactoring' stamp: ''!
refactor

	^ NautilusRefactoring model: self! !

!NautilusUI methodsFor: '*NautilusRefactoring' stamp: ''!
removeClassWithRefactoringEngine: aCollection

	^ self refactor removeClasses: aCollection ! !

!NautilusUI methodsFor: '*NautilusRefactoring' stamp: ''!
renameClassWithRefactoringEngine: aClass

	self refactor renameClass: aClass ! !

RBMethodRefactoring subclass: #RBCreateCascadeRefactoring
	instanceVariableNames: 'selector selectedInterval parseTree sequenceNode statementNodes transformedNode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NautilusRefactoring-Refactorings'!

!RBCreateCascadeRefactoring commentStamp: '<historical>' prior: 0!
A RBCreateCascadeRefactoring is a refactoring used to generate cascades in source code.

Copied from OB-Refactory-Tools and should be moved into RB instead
!

!RBCreateCascadeRefactoring class methodsFor: 'instance-creation' stamp: ''!
combine: anInterval from: aSelector in: aClass
	^ self new 
		combine: anInterval from: aSelector in: aClass;
		yourself! !

!RBCreateCascadeRefactoring class methodsFor: 'instance-creation' stamp: ''!
model: aNamespace combine: anInterval from: aSelector in: aClass
	^ self new 
		model: aNamespace; 
		combine: anInterval from: aSelector in: aClass;
		yourself! !

!RBCreateCascadeRefactoring methodsFor: 'preconditions' stamp: ''!
addStatementNode: aNode
	aNode isMessage
		ifTrue: [ ^ statementNodes add: aNode ].
	aNode isCascade
		ifTrue: [ ^ statementNodes addAll: aNode messages ].
	self refactoringError: aNode formattedCode , ' is not a valid message'! !

!RBCreateCascadeRefactoring methodsFor: 'initialization' stamp: ''!
combine: anInterval from: aSelector in: aClass
	class := self classObjectFor: aClass.
	selector := aSelector.
	selectedInterval := anInterval! !

!RBCreateCascadeRefactoring methodsFor: 'transforming' stamp: ''!
combineMessages
	"This combines the messages and adds the assignements of the last statement to the cascade. This is not necessary if there is a return, because the refactoring engine automatically compensates for that."

	| expression |
	transformedNode := RBCascadeNode messages: (statementNodes
		collect: [ :each | each copy ]).
	expression := statementNodes last parent.
	[ expression isAssignment ] whileTrue: [
		transformedNode := RBAssignmentNode
			variable: expression variable
			value: transformedNode.
		expression := expression parent ]! !

!RBCreateCascadeRefactoring methodsFor: 'transforming' stamp: ''!
compileCode
	class compileTree: (RBParseTreeRewriter
		replaceStatements: sequenceNode formattedCode
		with: transformedNode formattedCode
		in: self parseTree
		onInterval: selectedInterval)! !

!RBCreateCascadeRefactoring methodsFor: 'preconditions' stamp: ''!
findReceiverNode
	"Find the sequence to be combined."

	| receiverNodes |
	receiverNodes := statementNodes 
		collect: [ :each | each receiver ].
	receiverNodes asSet size = 1
		ifFalse: [ self refactoringError: 'All statements must have the same receiver' ].
	(receiverNodes first isLiteralNode or: [ receiverNodes first isVariable ])
		ifFalse: [ self refactoringWarning: 'The receiver is an expression. Proceed with caution' ]! !

!RBCreateCascadeRefactoring methodsFor: 'preconditions' stamp: ''!
findSequenceNode
	"Find the sequence to be combined."

	sequenceNode := RBParser
		parseExpression: self selectedSource
		onError: [ :msg :pos | self refactoringError: 'Invalid source to rewrite' ].
	(sequenceNode isSequence and: [ sequenceNode statements size > 1 ])
		ifFalse: [ self refactoringError: 'You must select two or more statements' ]! !

!RBCreateCascadeRefactoring methodsFor: 'preconditions' stamp: ''!
findStatementNodes
	"Find the sequence to be combined."

	statementNodes := OrderedCollection new.
	sequenceNode statements do: [ :each |
		(sequenceNode isLast: each)
			ifFalse: [ self addStatementNode: each ]
			ifTrue: [
				| current |
				current := each.
				[ current isReturn or: [ current isAssignment ] ]
					whileTrue: [ current := current value ].
				self addStatementNode: current ] ]! !

!RBCreateCascadeRefactoring methodsFor: 'accessing' stamp: ''!
parseTree
	parseTree isNil ifTrue: [
		parseTree := class parseTreeFor: selector.
		parseTree isNil
			ifTrue: [ self refactoringError: 'Could not parse sources' ] ].
	^ parseTree! !

!RBCreateCascadeRefactoring methodsFor: 'preconditions' stamp: ''!
preconditions
	^ (RBCondition definesSelector: selector in: class) & (RBCondition withBlock: [ 
		self findSequenceNode; findStatementNodes; findReceiverNode.
		true ])! !

!RBCreateCascadeRefactoring methodsFor: 'accessing' stamp: ''!
selectedSource
	^  self parseTree source copyFrom: selectedInterval first to: selectedInterval last! !

!RBCreateCascadeRefactoring methodsFor: 'transforming' stamp: ''!
transform
	self combineMessages.
	self compileCode! !

RBMethodRefactoring subclass: #RBSplitCascadeRefactoring
	instanceVariableNames: 'selector selectedInterval parseTree cascadeNode beforeNodes afterNodes ancestorNode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NautilusRefactoring-Refactorings'!

!RBSplitCascadeRefactoring class methodsFor: 'instance-creation' stamp: ''!
model: aNamespace split: anInterval from: aSelector in: aClass
	^ self new
		model: aNamespace;
		split: anInterval from: aSelector in: aClass;
		yourself! !

!RBSplitCascadeRefactoring class methodsFor: 'instance-creation' stamp: ''!
split: anInterval from: aSelector in: aClass
	^ self new 
		split: anInterval from: aSelector in: aClass;
		yourself! !

!RBSplitCascadeRefactoring methodsFor: 'transforming' stamp: ''!
extractReceiver
	| name |
	(cascadeNode receiver isLiteralNode or: [ cascadeNode receiver isVariable ])
		ifTrue: [ ^ self ].
	name := self 
		safeVariableNameFor: class 
		temporaries: self parseTree allDefinedVariables 
		basedOn: 'receiver'.
	ancestorNode parent
		addTemporaryNamed: name;
		addNode: (RBAssignmentNode 
			variable: (RBVariableNode named: name)
			value: cascadeNode receiver)
		before: ancestorNode.
	cascadeNode messages
		do: [ :each | each receiver: (RBVariableNode named: name) ]
		! !

!RBSplitCascadeRefactoring methodsFor: 'preconditions' stamp: ''!
findAncestorNode
	"The ancestor node is the node that is contained within the sequence. In most cases this is the cascade itself, but it also can be an assignment or a return node."

	ancestorNode := cascadeNode.
	[ ancestorNode parent isSequence not and: [ ancestorNode parent isAssignment ] ]
		whileTrue: [ ancestorNode := ancestorNode parent ].
	[ ancestorNode parent isSequence not and: [ ancestorNode parent isReturn ] ]
		whileTrue: [ ancestorNode := ancestorNode parent ].
	ancestorNode parent isSequence
		ifFalse: [ self refactoringError: 'To split this cascade, you must extract it to a temporary first' ]! !

!RBSplitCascadeRefactoring methodsFor: 'preconditions' stamp: ''!
findCascadeNode
	"Find the cascade to be split."

	cascadeNode := self parseTree bestNodeFor: selectedInterval.
	[ cascadeNode isNil or: [ cascadeNode isCascade ] ]
		whileFalse: [ cascadeNode := cascadeNode parent ].
	cascadeNode isNil
		ifTrue: [ self refactoringError: 'The selection doesn''t appear to be within a cascade' ]! !

!RBSplitCascadeRefactoring methodsFor: 'preconditions' stamp: ''!
findMessageNodes
	"Find the nodes that form the first part of the cascade and the second part of the cascade."

	beforeNodes := cascadeNode messages
		select: [ :each | each stop <= selectedInterval first ].
	afterNodes := cascadeNode messages
		select: [ :each | selectedInterval last <= each selectorParts first start ].
	(beforeNodes isEmpty or: [ afterNodes isEmpty ])
		ifTrue: [ self refactoringError: 'Splitting a cascade into the whole cascade and an empty one is pointless' ].
	(beforeNodes size + afterNodes size = cascadeNode messages size)
		ifFalse: [ self refactoringError: 'To set the split boundary place the cursor inbetween two cascaded messages' ]! !

!RBSplitCascadeRefactoring methodsFor: 'accessing' stamp: ''!
parseTree
	parseTree isNil ifTrue: [
		parseTree := class parseTreeFor: selector.
		parseTree isNil
			ifTrue: [ self refactoringError: 'Could not parse sources' ] ].
	^ parseTree! !

!RBSplitCascadeRefactoring methodsFor: 'preconditions' stamp: ''!
preconditions
	^ (RBCondition definesSelector: selector in: class) & (RBCondition withBlock: [ 
		self findCascadeNode; findAncestorNode; findMessageNodes. 
		true ])! !

!RBSplitCascadeRefactoring methodsFor: 'initialization' stamp: ''!
split: anInterval from: aSelector in: aClass
	class := self classObjectFor: aClass.
	selector := aSelector.
	selectedInterval := anInterval! !

!RBSplitCascadeRefactoring methodsFor: 'transforming' stamp: ''!
splitCascade
	ancestorNode parent
		addNode: (beforeNodes size > 1
			ifTrue: [ RBCascadeNode messages: beforeNodes ]
			ifFalse: [ beforeNodes first ])
		before: ancestorNode.
	afterNodes size > 1
		ifTrue: [ cascadeNode messages: afterNodes ]
		ifFalse: [ cascadeNode replaceWith: afterNodes first ].
	class compileTree: ancestorNode methodNode! !

!RBSplitCascadeRefactoring methodsFor: 'transforming' stamp: ''!
transform
	self extractReceiver.
	self splitCascade! !

RBMethodRefactoring subclass: #RBSwapMethodRefactoring
	instanceVariableNames: 'target selector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NautilusRefactoring-Refactorings'!

!RBSwapMethodRefactoring commentStamp: 'lr 10/19/2007 09:16' prior: 0!
Move a method from the class to the instance side, or vice versa. Normally this is not considered to be a refactoring.!

!RBSwapMethodRefactoring class methodsFor: 'instance-creation' stamp: ''!
model: aRBSmalltalk swapMethod: aSelector in: aClass
	^ self new 
		model: aRBSmalltalk;
		swapMethod: aSelector in: aClass;
		yourself! !

!RBSwapMethodRefactoring class methodsFor: 'instance-creation' stamp: ''!
swapMethod: aSelector in: aClass
	^ self new swapMethod: aSelector in: aClass! !

!RBSwapMethodRefactoring methodsFor: 'preconditions' stamp: ''!
checkInstVars
	class instanceVariableNames do: [ :each |
		(target instanceVariableNames includes: each) ifFalse: [
			((class whichSelectorsReferToInstanceVariable: each) includes: selector) ifTrue: [
				self
					refactoringError: ('<1p> refers to <2s>, which is not defined in <3p>' expandMacrosWith: selector with: each with: target) ] ] ]! !

!RBSwapMethodRefactoring methodsFor: 'preconditions' stamp: ''!
preconditions
	^ (RBCondition definesSelector: selector in: class)
		& (RBCondition definesSelector: selector in: target) not
		& (RBCondition withBlock: [ self checkInstVars. true ])! !

!RBSwapMethodRefactoring methodsFor: 'initialization' stamp: ''!
swapMethod: aSelector in: aClass 
	class := self classObjectFor: aClass.
	target := self classObjectFor: (class isMeta 
			ifTrue: [ class theNonMetaClass ]
			ifFalse: [ class theMetaClass ]).
	selector := aSelector! !

!RBSwapMethodRefactoring methodsFor: 'transforming' stamp: ''!
transform
	target compile: (class sourceCodeFor: selector) classified: (class protocolsFor: selector).
	class removeMethod: selector! !

DialogWindow subclass: #MethodNameEditor
	instanceVariableNames: 'argumentIndex labelMorph methodName selectorField'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NautilusRefactoring-Utilities'!

!MethodNameEditor commentStamp: '<historical>' prior: 0!
A MethodNameEditor is an editor to set the name of a method.
This class is imported from OB.

Maybe it should be redone with Spec, but since it works well, I will not put more energy here.!

!MethodNameEditor class methodsFor: 'instance creation' stamp: ''!
on: aMethodName
	^ self basicNew initializeOn: aMethodName! !

!MethodNameEditor class methodsFor: 'instance creation' stamp: ''!
openOn: aMethodName
	^ UITheme builder openModal: (self on: aMethodName)! !

!MethodNameEditor methodsFor: 'grips' stamp: 'BenjaminVanRyseghem 4/25/2012 13:54'!
addEdgeGrips
	"Should add these to the front!!"
	
	| l r lh |
	lh := self labelHeight.
	l  := WindowEdgeGripMorph new target: self; position: self position; edgeName: #left.
	l layoutFrame topOffset: lh negated + 22.
	r  := WindowEdgeGripMorph new target: self; position: self position; edgeName: #right.
	r layoutFrame topOffset: lh negated + 22.
	self
		addMorph: l;
		addMorph: r
! !

!MethodNameEditor methodsFor: 'grips' stamp: 'BenjaminVanRyseghem 4/25/2012 13:53'!
addGrips

	self addEdgeGrips! !

!MethodNameEditor methodsFor: 'accessing' stamp: ''!
argumentIndex
	^ argumentIndex! !

!MethodNameEditor methodsFor: 'accessing' stamp: ''!
argumentIndex: anInteger
	argumentIndex := anInteger.
	self update! !

!MethodNameEditor methodsFor: 'accessing' stamp: ''!
argumentList
	^ self methodName arguments! !

!MethodNameEditor methodsFor: 'actions' stamp: ''!
cancel
	methodName := nil.
	^ super cancel! !

!MethodNameEditor methodsFor: 'focus handling' stamp: 'BenjaminVanRyseghem 3/26/2012 17:10'!
defaultFocusMorph
	"Answer the morph that should have the keyboard
	focus by default when the dialog is opened."
	
	^ selectorField
		ifNil: [ super defaultFocusMorph ]! !

!MethodNameEditor methodsFor: 'actions' stamp: ''!
down
	self isDownEnabled ifFalse: [ ^ self ].
	self argumentList swap: self argumentIndex with: self argumentIndex + 1.
	self argumentIndex: self argumentIndex + 1! !

!MethodNameEditor methodsFor: 'initialize' stamp: 'BenjaminVanRyseghem 4/25/2012 13:39'!
initialize
	"Initialization code for MethodNameEditor"

	super initialize.

	self beResizeable! !

!MethodNameEditor methodsFor: 'initialization' stamp: ''!
initializeOn: aMethodName
	methodName := aMethodName.
	argumentIndex := 0.
	self initialize.
	self title: 'Method Name'! !

!MethodNameEditor methodsFor: 'testing' stamp: ''!
isDownEnabled
	^ self argumentIndex ~= 0 and: [ self argumentIndex + 1 between: 1 and: self argumentList size ]! !

!MethodNameEditor methodsFor: 'testing' stamp: ''!
isOkEnabled
	^ self methodName isValid! !

!MethodNameEditor methodsFor: 'testing' stamp: ''!
isUpEnabled
	^ self argumentIndex ~= 0 and: [ self argumentIndex - 1 between: 1 and: self argumentList size ]! !

!MethodNameEditor methodsFor: 'accessing' stamp: ''!
methodName
	^ methodName! !

!MethodNameEditor methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 3/26/2012 17:19'!
newContentMorph
	^ (self newRow: {
		self newLabelGroup: {
			'Selector:' -> (selectorField:=(self newTextEntryFor: self getText: #selector setText: #selector: help: nil)
				autoAccept: true;
				on: #keyStroke send: #value:value: to: [:key :morph | key keyCharacter = Character cr ifTrue: [ self ok. true]. false];
				yourself).
			'Arguments:' -> (self newRow: {
				(self newListFor: self list: #argumentList selected: #argumentIndex changeSelected: #argumentIndex: help: nil)
					hResizing: #spaceFill;
					yourself.
				(self newColumn: {
					(self newButtonFor: self action: #up label: 'up'  help: nil)
						getEnabledSelector: #isUpEnabled.
					(self newButtonFor: self action: #down label: 'dn'  help: nil)
						getEnabledSelector: #isDownEnabled })
					hResizing: #shrinkWrap }).
			'Preview:' -> (labelMorph := self newLabel: self methodName printString) } })
 		minWidth: 400;
		yourself! !

!MethodNameEditor methodsFor: 'morphic' stamp: ''!
newOKButton
	^ self newOKButtonFor: self getEnabled: #isOkEnabled! !

!MethodNameEditor methodsFor: 'accessing' stamp: ''!
selector
	^ self methodName selector! !

!MethodNameEditor methodsFor: 'accessing' stamp: ''!
selector: aString
	self methodName selector: aString.
	self update! !

!MethodNameEditor methodsFor: 'actions' stamp: ''!
up
	self isUpEnabled ifFalse: [ ^ self ].
	self argumentList swap: self argumentIndex with: self argumentIndex - 1.
	self argumentIndex: self argumentIndex - 1! !

!MethodNameEditor methodsFor: 'morphic' stamp: ''!
update
	self changed: #argumentList; changed: #argumentIndex.
	self changed: #isUpEnabled; changed: #isDownEnabled; changed: #isOkEnabled.
	labelMorph contents: self methodName printString! !

RBClassRefactoring subclass: #RBAccessorClassRefactoring
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NautilusRefactoring-Refactorings'!

!RBAccessorClassRefactoring methodsFor: 'preconditions' stamp: ''!
preconditions
	^ self refactorings
		inject: RBCondition empty
		into: [ :result :each | result & each preconditions ]! !

!RBAccessorClassRefactoring methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 4/25/2012 18:02'!
refactorings
	| class |
	class := self classObjectFor: className asSymbol.
	^  class instanceVariableNames collect: [ :each | 
		RBCreateAccessorsForVariableRefactoring
			variable: each class: class classVariable: false ]! !

!RBAccessorClassRefactoring methodsFor: 'transforming' stamp: ''!
transform
	self refactorings do: [ :each | self performComponentRefactoring: each ]! !

RBClassRefactoring subclass: #RBGenerateEqualHashRefactoring
	instanceVariableNames: 'variables'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NautilusRefactoring-Refactorings'!

!RBGenerateEqualHashRefactoring class methodsFor: 'instance-creation' stamp: ''!
className: aClass variables: anArray
	^ (self className: aClass) variables: anArray! !

!RBGenerateEqualHashRefactoring class methodsFor: 'instance-creation' stamp: ''!
model: aNamespace className: aClass variables: anArray
	^ (self model: aNamespace className: aClass) variables: anArray! !

!RBGenerateEqualHashRefactoring methodsFor: 'transforming' stamp: 'BenjaminVanRyseghem 4/25/2012 18:02'!
accessorForVariable: aString
	| refactoring |
	refactoring := RBCreateAccessorsForVariableRefactoring
		model: self model
		variable: aString
		class: self theClass
		classVariable: false.
	refactoring createGetterAccessor.
	^ refactoring getterMethod! !

!RBGenerateEqualHashRefactoring methodsFor: 'transforming' stamp: ''!
compileEqual
	| method statement comparison |
	method := RBParser parseMethod: '= anObject
		"Answer whether the receiver and anObject represent the same object."

		self == anObject ifTrue: [ ^ true ].
		self class = anObject class ifFalse: [ ^ false ]'.
	statement := nil.	
	variables reversed do: [ :each |
		| accessor |
		accessor := self accessorForVariable: each.
		comparison := RBMessageNode 
			receiver: (RBVariableNode named: each) 
			selector: #=
			arguments: (Array with: (RBMessageNode 
				receiver: (RBVariableNode named: 'anObject')
				selector: accessor)).
		statement := statement isNil
			ifTrue: [ comparison ]
			ifFalse: [
				RBMessageNode
					receiver: comparison
					selector: #and:
					arguments: (Array with: (RBBlockNode 
						body: (RBSequenceNode 
							statements: (Array with: statement)))) ] ].
	method addNode: statement; addReturn.
	self theClass 
		compile: method formattedCode
		classified: #(comparing)! !

!RBGenerateEqualHashRefactoring methodsFor: 'transforming' stamp: ''!
compileHash
	| method statement hash |
	method := RBParser parseMethod: 'hash
		"Answer an integer value that is related to the identity of the receiver."'.
	statement := nil.
	variables reversed do: [ :each |
		hash := RBMessageNode 
			receiver: (RBVariableNode named: each)
			selector: #hash.
		statement := statement isNil
			ifTrue: [ hash ]
			ifFalse: [
				RBMessageNode
					receiver: hash
					selector: #bitXor:
					arguments: (Array with: statement) ] ].
	method addNode: statement; addReturn.
	self theClass
		compile: method formattedCode
		classified: #(comparing)! !

!RBGenerateEqualHashRefactoring methodsFor: 'preconditions' stamp: ''!
preconditions
	^ variables
		inject: RBCondition empty
		into: [ :condition :variable | condition & (RBCondition definesInstanceVariable: variable in: self theClass) ]! !

!RBGenerateEqualHashRefactoring methodsFor: 'accessing' stamp: ''!
theClass
	^ (self classObjectFor: className) theNonMetaClass! !

!RBGenerateEqualHashRefactoring methodsFor: 'transforming' stamp: ''!
transform
	self compileHash.
	self compileEqual! !

!RBGenerateEqualHashRefactoring methodsFor: 'accessing' stamp: ''!
variables: anArray
	variables := anArray! !

RBClassRefactoring subclass: #RBGeneratePrintStringRefactoring
	instanceVariableNames: 'variables'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NautilusRefactoring-Refactorings'!

!RBGeneratePrintStringRefactoring class methodsFor: 'instance-creation' stamp: ''!
className: aClass variables: anArray
	^ (self className: aClass) variables: anArray! !

!RBGeneratePrintStringRefactoring class methodsFor: 'instance-creation' stamp: ''!
model: aNamespace className: aClass variables: anArray
	^ (self model: aNamespace className: aClass) variables: anArray! !

!RBGeneratePrintStringRefactoring methodsFor: 'preconditions' stamp: ''!
preconditions
	^ variables
		inject: RBCondition empty
		into: [ :condition :variable | condition & (RBCondition definesInstanceVariable: variable in: self theClass) ]! !

!RBGeneratePrintStringRefactoring methodsFor: 'accessing' stamp: ''!
theClass
	^ (self classObjectFor: className) theNonMetaClass! !

!RBGeneratePrintStringRefactoring methodsFor: 'transforming' stamp: ''!
transform
	| method |
	method := RBParser parseMethod: 'printOn: aStream
		"Append a sequence of characters to aStream that identify the receiver."
		
		super printOn: aStream'.
	variables do: [ :each |
		method body addNode: (RBParser 
			parseExpression: ('aStream nextPutAll: '' <1s>: ''; print: <1s>'
				expandMacrosWith: each)) ].
	self theClass
		compile: method formattedCode
		classified: #(printing)! !

!RBGeneratePrintStringRefactoring methodsFor: 'accessing' stamp: ''!
variables: anArray
	variables := anArray! !

RBClassRefactoring subclass: #RBRealizeClassRefactoring
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NautilusRefactoring-Refactorings'!

!RBRealizeClassRefactoring commentStamp: 'lr 10/19/2007 09:16' prior: 0!
Make a given class concrete, by providing empty templates for all the abstract methods.!

!RBRealizeClassRefactoring methodsFor: 'preconditions' stamp: ''!
preconditions
	^ RBCondition
		withBlock: [
			(self theClass withAllSubclasses
				detect: [ :each | 
					(each whichSelectorsReferToSymbol: #subclassResponsibility) notEmpty 
						or: [ (each theMetaClass whichSelectorsReferToSymbol: #subclassResponsibility) notEmpty ] ]
				ifNone: [ nil ]) isNil ]
		errorString: self theClass printString , ' is abstract or has abstract subclasses.'! !

!RBRealizeClassRefactoring methodsFor: 'accessing' stamp: ''!
theClass
	^ (self classObjectFor: className) theNonMetaClass! !

!RBRealizeClassRefactoring methodsFor: 'transforming' stamp: ''!
transform
	self transform: self theClass.
	self transform: self theClass theMetaClass! !

!RBRealizeClassRefactoring methodsFor: 'transforming' stamp: ''!
transform: aClass
	| class method parseTree |
	aClass allSelectors do: [ :selector |
		class := aClass whoDefinesMethod: selector.
		(class notNil and: [ class ~= aClass ]) ifTrue: [
			method := class methodFor: selector.
			(method notNil and: [ method refersToSymbol: #subclassResponsibility ]) ifTrue: [
				parseTree := method parseTree.
				parseTree body 
					temporaries: OrderedCollection new; 
					statements: OrderedCollection new;
					addNode: (RBMessageNode 
						receiver: (RBVariableNode named: 'self')
						selector: #shouldBeImplemented).
				aClass compile: parseTree newSource withAttributesFrom: method ] ] ]! !

!RBCreateAccessorsForVariableRefactoring methodsFor: '*NautilusRefactoring' stamp: ''!
whatToDisplayIn: aBrowser 

	^ (self changes changes select: [:change | {getterMethod. setterMethod} includes: change selector ]) gather: [:change | change whatToDisplayIn: aBrowser ]! !

!RBCompositeRefactoryChange methodsFor: '*NautilusRefactoring' stamp: ''!
whatToDisplayIn: aBrowser

	^ self changes gather: [:each | each whatToDisplayIn: aBrowser ]! !

!RBAddMethodChange methodsFor: '*NautilusRefactoring' stamp: ''!
accept: aText notifying: aController
	"Just to make sure that it compiles, try with the standard compiler."
	
	| compilerClass |
	compilerClass := self changeClass
		ifNil: [ Object compilerClass ]
		ifNotNil: [:changeClass | changeClass compilerClass ].
		
	compilerClass new
		compile: aText asString in: self changeClass
		classified: nil notifying: aController
		ifFail: [ ^ false ].
		
	self 
		class: self changeClass
		protocol: self protocol
		source: aText asString.
		
	^ true! !

!RBAddMethodChange methodsFor: '*NautilusRefactoring' stamp: ''!
oldVersionTextToDisplay

	| class |
	class := Smalltalk 
				at: className asSymbol
				ifAbsent: [ ^ super oldVersionTextToDisplay ].
				
	^ class methodDict 
			at: self selector
			ifPresent: [:method | method sourceCode ]
			ifAbsent: [ super oldVersionTextToDisplay ]! !

!RBAddMethodChange methodsFor: '*NautilusRefactoring' stamp: ''!
textToDisplay

	^ self source! !

!RBRenameVariableChange methodsFor: '*NautilusRefactoring' stamp: ''!
nameToDisplay

	^ 'Rename ', self oldName, ' into ', self newName! !

!RBRenameVariableChange methodsFor: '*NautilusRefactoring' stamp: ''!
textToDisplay

	self printString! !

!RBRenameVariableChange methodsFor: '*NautilusRefactoring' stamp: ''!
whatToDisplayIn: aChangeBrowser

	| result |
	result := OrderedCollection with: self.
	^ result, (self changes gather: [:e | e changes ])! !

RBRefactoring subclass: #RBPrettyPrintCodeRefactoring
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NautilusRefactoring-Refactorings'!

!RBPrettyPrintCodeRefactoring methodsFor: 'preconditions' stamp: ''!
preconditions
	^ RBCondition empty! !

!RBPrettyPrintCodeRefactoring methodsFor: 'transforming' stamp: ''!
transform
	| source tree formatted |
	self model allClassesDo: [ :class |
		class selectors do: [ :selector |
			(self model environment includesSelector: selector in: class realClass) ifTrue: [
				source := class sourceCodeFor: selector.
				source isNil ifFalse: [ 
					tree := class parseTreeFor: selector.
					tree isNil ifFalse: [
						formatted := tree formattedCode.
						(source ~= formatted and: [ (RBParser parseMethod: formatted) = tree ]) 
							ifTrue: [ class compile: formatted classified: (class protocolsFor: selector) ] ] ] ] ] ] ! !

!RBRefactoring methodsFor: '*NautilusRefactoring' stamp: ''!
whatToDisplayIn: aBrowser 
	
	^ self changes changes gather: [:e | e whatToDisplayIn: aBrowser ]! !

RBRefactoring subclass: #RBRegexRefactoring
	instanceVariableNames: 'matchers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NautilusRefactoring-Refactorings'!

RBRegexRefactoring subclass: #RBCategoryRegexRefactoring
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NautilusRefactoring-Refactorings'!

!RBCategoryRegexRefactoring methodsFor: 'transforming' stamp: ''!
transform
	| replacement |
	self model allClassesDo: [ :class |
		(class isNil or: [ class isMeta ]) ifFalse: [
			replacement := self execute: class category.
			replacement = class category asString ifFalse: [
				class category: replacement.
				self model defineClass: class definitionString ] ] ]! !

RBRegexRefactoring subclass: #RBClassRegexRefactoring
	instanceVariableNames: 'rootClass mode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NautilusRefactoring-Refactorings'!

!RBClassRegexRefactoring methodsFor: 'transforming' stamp: ''!
copy: aClass name: aSymbol
	^ self duplicate: aClass name: aSymbol deep: true! !

!RBClassRegexRefactoring methodsFor: 'actions' stamp: ''!
copyClasses
	mode := #copy:name:! !

!RBClassRegexRefactoring methodsFor: 'private' stamp: ''!
copyFrom: aSourceClass to: aTargetClass
	aSourceClass instanceVariableNames
		do: [ :each | aTargetClass addInstanceVariable: each ].
	aSourceClass isMeta ifFalse: [
		aSourceClass allClassVariableNames
			do: [ :each | aTargetClass addClassVariable: each ].
		aSourceClass poolDictionaryNames
			do: [ :each | aTargetClass addPoolDictionary: each ] ].
	aSourceClass selectors do: [ :each |
		aTargetClass
			compile: (aSourceClass sourceCodeFor: each)
			classified: (aSourceClass protocolsFor: each) ]! !

!RBClassRegexRefactoring methodsFor: 'transforming' stamp: ''!
create: aClass name: aSymbol
	^ self duplicate: aClass name: aSymbol deep: false! !

!RBClassRegexRefactoring methodsFor: 'actions' stamp: ''!
createClasses
	mode := #create:name:! !

!RBClassRegexRefactoring methodsFor: 'private' stamp: ''!
duplicate: aClass name: aSymbol deep: aBoolean
	| superclass superclassName name class |
	(self model includesClassNamed: aSymbol)
		ifTrue: [ ^ nil ].
	superclass := aClass superclass 
		ifNil: [ self rootClass ].
	superclassName := (self model includesClassNamed: superclass name)
		ifFalse: [ superclass name ]
		ifTrue: [ 
			(name := self execute: superclass name) = superclass name
				ifFalse: [ self duplicate: superclass name: name deep: aBoolean ].
			name ].
	self model defineClass: ('<1s>
		subclass: #<2s>
		instanceVariableNames: ''''
		classVariableNames: ''''
		poolDictionaries: ''''
		category: <3p>' 
			expandMacrosWith: superclassName 
			with: aSymbol with: aClass category asString).
	aBoolean ifTrue: [
		(class := self model classNamed: aSymbol) 
			ifNil: [ ^ self ].
		self copyFrom: aClass to: class.
		self copyFrom: aClass theMetaClass to: class theMetaClass ].
	^ nil! !

!RBClassRegexRefactoring methodsFor: 'initialization' stamp: ''!
initialize
	super initialize.
	self createClasses! !

!RBClassRegexRefactoring methodsFor: 'transforming' stamp: 'BenjaminVanRyseghem 4/25/2012 18:05'!
rename: aClass name: aSymbol
	^ RBRenameClassRefactoring 
		model: self model
		rename: aClass
		to: aSymbol! !

!RBClassRegexRefactoring methodsFor: 'actions' stamp: ''!
renameClasses
	mode := #rename:name:! !

!RBClassRegexRefactoring methodsFor: 'accessing' stamp: ''!
rootClass
	^ rootClass ifNil: [ Object ]! !

!RBClassRegexRefactoring methodsFor: 'initialization' stamp: ''!
rootClass: aClass
	rootClass := aClass theNonMetaClass! !

!RBClassRegexRefactoring methodsFor: 'transforming' stamp: ''!
transform
	| replacement refactoring |
	self model allClassesDo: [ :class |
		(class isNil or: [ class isMeta ]) ifFalse: [
			replacement := self execute: class name.
			replacement = class name asString ifFalse: [
				refactoring := self perform: mode
					with: class with: replacement asSymbol.
				(refactoring notNil and: [ refactoring preconditions check ])
					ifTrue: [ refactoring transform ] ] ] ]! !

RBRegexRefactoring subclass: #RBProtocolRegexRefactoring
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NautilusRefactoring-Refactorings'!

!RBProtocolRegexRefactoring methodsFor: 'transforming' stamp: ''!
transform
	| original replacement |
	self model allClassesDo: [ :class |
		class selectors do: [ :selector |
			original := (class realClass whichCategoryIncludesSelector: selector)
				asString.
			original isNil ifFalse: [
				replacement := self execute: original.
				replacement = original ifFalse: [
					class
						compile: (class sourceCodeFor: selector)
						classified: replacement ] ] ] ]! !

!RBRegexRefactoring methodsFor: 'private' stamp: ''!
execute: aString
	"Perform all searches on aString and return the transformation."
	
	^ matchers 
		inject: aString asString
		into: [ :string :assoc | self execute: string replace: assoc key with: assoc value ]! !

!RBRegexRefactoring methodsFor: 'private' stamp: ''!
execute: aString replace: aRegex with: aReadStream
	| stream |
	^ aRegex copy: aString translatingMatchesUsing: [ :match |
		stream := WriteStream on: (String new: 2 * aString size).
		[ aReadStream atEnd ] whileFalse: [
			stream nextPutAll: (aReadStream upTo: $$).
			aReadStream atEnd ifFalse: [
				aReadStream peek isDigit
					ifFalse: [ stream nextPut: aReadStream next ]
					ifTrue: [ stream nextPutAll: (aRegex subexpression: aReadStream next asInteger - $0 asInteger + 1) ] ] ].
		aReadStream reset.
		stream contents ]! !

!RBRegexRefactoring methodsFor: 'initialize' stamp: ''!
initialize
	super initialize.
	matchers := OrderedCollection new! !

!RBRegexRefactoring methodsFor: 'preconditions' stamp: ''!
preconditions
	^ RBCondition empty! !

!RBRegexRefactoring methodsFor: 'searching' stamp: ''!
replace: aFindString with: aReplaceString
	self replace: aFindString with: aReplaceString ignoreCase: false! !

!RBRegexRefactoring methodsFor: 'searching' stamp: ''!
replace: aFindString with: aReplaceString ignoreCase: aBoolean
	"Replace all matches of aFindString (regular expression) with aReplaceString, where $0 references the whole match, and $1..$9 the matched groups."
	
	| regex stream |
	regex := RxParser preferredMatcherClass
		for: (RxParser new parse: aFindString)
		ignoreCase: aBoolean.
	stream := aReplaceString readStream.
	matchers add: regex -> stream! !

RBRegexRefactoring subclass: #RBSourceRegexRefactoring
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NautilusRefactoring-Refactorings'!

!RBSourceRegexRefactoring methodsFor: 'private' stamp: ''!
parseMethod: aString
	^ [ RBParser parseMethod: aString ]
		on: Error do: [ :err | nil ]! !

!RBSourceRegexRefactoring methodsFor: 'private' stamp: ''!
parseSelector: aString
	^ RBParser parseMethodPattern: aString! !

!RBSourceRegexRefactoring methodsFor: 'transforming' stamp: ''!
transform
	| original replacement protocols |
	self model allClassesDo: [ :class |
		class selectors do: [ :selector |
			original := class sourceCodeFor: selector.
			replacement := self execute: original.
			replacement = original ifFalse: [
				(self parseMethod: replacement) isNil ifFalse: [
					protocols := class protocolsFor: selector.
					(self parseSelector: replacement) = selector
						ifFalse: [ class removeMethod: selector ].
					class compile: replacement classified: protocols ] ] ] ]! !

!RBRefactoryChange methodsFor: '*NautilusRefactoring' stamp: ''!
accept: aText notifying: aController
	^ false! !

!RBRefactoryChange methodsFor: '*NautilusRefactoring' stamp: ''!
nameToDisplay

	^ self name! !

!RBRefactoryChange methodsFor: '*NautilusRefactoring' stamp: ''!
oldVersionTextToDisplay

	^ ''! !

!RBRefactoryChange methodsFor: '*NautilusRefactoring' stamp: ''!
textToDisplay

	^ self name! !

!RBRefactoryChange methodsFor: '*NautilusRefactoring' stamp: ''!
whatToDisplayIn: aChangeBrowser

	^ { self }! !

!RBRenameClassChange methodsFor: '*NautilusRefactoring' stamp: ''!
nameToDisplay

	^ self printString! !

!RBRenameClassChange methodsFor: '*NautilusRefactoring' stamp: ''!
textToDisplay

	^ self printString! !

!RBRenameClassChange methodsFor: '*NautilusRefactoring' stamp: ''!
whatToDisplayIn: aChangeBrowser

	| result |
	result := OrderedCollection with: self.
	^ result, (self changes gather: [:e | e changes ])! !

!RBAddClassChange methodsFor: '*NautilusRefactoring' stamp: ''!
nameToDisplay

	^ self changeString! !

!RBAddClassChange methodsFor: '*NautilusRefactoring' stamp: ''!
textToDisplay

	^ self definition! !

!RGMethodDefinition methodsFor: '*NautilusRefactoring' stamp: ''!
nameToDisplay

	^ self printString! !

!RBChildrenToSiblingsRefactoring methodsFor: '*NautilusRefactoring' stamp: ''!
whatToDisplayIn: aChangeBrowser

	^ aChangeBrowser changesOf: self! !

Object subclass: #NautilusRefactoring
	instanceVariableNames: 'environment model'
	classVariableNames: 'PromptOnRefactoring'
	poolDictionaries: ''
	category: 'NautilusRefactoring'!

!NautilusRefactoring commentStamp: '<historical>' prior: 0!
NautilusRefactoring is a facade for refactorings:
- class
- method
- inst var
- class var
- source code!

!NautilusRefactoring class methodsFor: 'shortcuts' stamp: 'BenjaminVanRyseghem 4/18/2012 16:12'!
buildRefactoringMethodShortcutsOn: aBuilder
	<keymap>	

	(aBuilder shortcut: #rename)
		category: #NautilusMethodShortcuts
		default: $r command
		do: [:target || scroll |
				scroll := target methodWidget vScrollValue.
				target refactor renameMethodFor: target selectedMethod.
				target methodWidget vScrollValue: scroll]
		description: 'Rename the seleted method'
		! !

!NautilusRefactoring class methodsFor: 'shortcuts' stamp: 'BenjaminVanRyseghem 4/18/2012 17:44'!
buildSourceCodeShortcutsOn: aBuilder
	<keymap>	

	(aBuilder shortcut: #rename)
		category: #NautilusSourceCodeShortcuts
		default: $r command
		do: [:target | (target sourceTextArea hasUnacceptedEdits not)
							ifTrue: [ target refactor renameTextSelection ]]
		description: 'Rename the current selection'.
		
	(aBuilder shortcut: #extractToTemp)
		category: #NautilusSourceCodeShortcuts
		default: $x command shift
		do: [:target | (target sourceTextArea hasUnacceptedEdits not)
							ifTrue: [ target refactor extractToTempTextSelection ]]
		description: 'Extract to temp the current selection'.
		
	(aBuilder shortcut: #extractToMethod)
		category: #NautilusSourceCodeShortcuts
		default: $m command shift
		do: [:target | (target sourceTextArea hasUnacceptedEdits not)
							ifTrue: [ target refactor extractToMethodTextSelection ]]
		description: 'Extract to method the current selection'.! !

!NautilusRefactoring class methodsFor: 'menu' stamp: 'BenjaminVanRyseghem 5/13/2012 17:54'!
classRefactoringSubmenu: aBuilder
	<nautilusClassMenu>
	| target |
	target := aBuilder model.
	target selectedClass ifNil: [^target].
	
	(aBuilder item: #'Rename... (r)')
		action: [ target renameClassWithRefactoringEngine: target selectedClass ];
		parent: #'Class Refactoring';
		order: 0.
	
	(aBuilder item: #'Remove...')
		action: [ target removeClassWithRefactoringEngine: target selectedClasses ];
		icon: (target iconClass iconNamed: #removeIcon);
		parent: #'Class Refactoring';
		order: 100;
		withSeparatorAfter.
		
	(aBuilder item: #'Generate Accessors')
		action: [ target refactor generateAccessors ];
		parent: #'Class Refactoring';
		order: 200.
		
	(aBuilder item: #'Generate Superclass...')
		action: [ target refactor generateSuperClass ];
		parent: #'Class Refactoring';
		order: 300.
		
	(aBuilder item: #'Generate Subclass...')
		action: [ target refactor generateSubclass ];
		parent: #'Class Refactoring';
		order: 400.
		
	(aBuilder item: #'Realize')
		action: [ target refactor realizeClass ];
		parent: #'Class Refactoring';
		order: 500.
		
	(aBuilder item: #'Split')
		action: [ target refactor splitClass: target selectedClass ];
		parent: #'Class Refactoring';
		order: 600.! !

!NautilusRefactoring class methodsFor: 'menu' stamp: 'BenjaminVanRyseghem 5/13/2012 17:56'!
classVarRefactoringSubmenu: aBuilder
	<nautilusClassMenu>
	| target selectedClass |
	target := aBuilder model.
	selectedClass := target selectedClass.
	selectedClass ifNil: [ ^ target ].
	
	(aBuilder item: #'Add')
		action: [ target refactor addClassVarFrom: selectedClass ];
		parent: #'Class Var Refactoring';
		order: 0.
		
	(aBuilder item: #'Rename')
		action: [ target refactor renameClassVarFrom: selectedClass ];
		parent: #'Class Var Refactoring';
		order: 100.
	
	(aBuilder item: #'Remove')
		action: [ target refactor removeClassVarFrom: selectedClass ];
		parent: #'Class Var Refactoring';
		icon: (target iconClass iconNamed: #removeIcon);
		order: 200;
		withSeparatorAfter.
		
	(aBuilder item: #'Abstract')
		action: [ target refactor abstractClassVarFrom: selectedClass ];
		parent: #'Class Var Refactoring';
		order: 300.
	
	(aBuilder item: #'Accessors')
		action: [ target refactor accessorsClassVarFrom: selectedClass ];
		parent: #'Class Var Refactoring';
		order: 400.
		
	(aBuilder item: #'Pull up')
		action: [ target refactor pullUpClassVarFrom: selectedClass ];
		parent: #'Class Var Refactoring';
		order: 500.
	
	(aBuilder item: #'Push down')
		action: [ target refactor pushDownClassVarFrom: selectedClass ];
		parent: #'Class Var Refactoring';
		order: 600.! !

!NautilusRefactoring class methodsFor: 'menu' stamp: 'BenjaminVanRyseghem 5/13/2012 17:54'!
codeRewritingClassSubmenu: aBuilder
	<nautilusClassMenu>
	| target |
	target := aBuilder model.
	target selectedClass ifNil: [^target].
	
	(aBuilder item: #'Rewrite Code')
		action: [ target refactor rewriteCode ];
		parent: #'Code Rewriting';
		order: 0.

	(aBuilder item: #'Search Code')
		action: [ target refactor searchCode ];
		parent: #'Code Rewriting';
		order: 100.
		
	(aBuilder item: #'Type class')
		action: [ target refactor typeClass ];
		parent: #'Code Rewriting';
		order: 200;
		withSeparatorAfter.
		
	(aBuilder item: #'Category Regex')
		action: [ target refactor categoryRegex ];
		parent: #'Code Rewriting';
		order: 300.
		
	(aBuilder item: #'Class Regex')
		action: [ target refactor classRegex ];
		parent: #'Code Rewriting';
		order: 400.
		
	(aBuilder item: #'Protocol Regex')
		action: [ target refactor protocolRegex ];
		parent: #'Code Rewriting';
		order: 500.
		
	(aBuilder item: #'Source Regex')
		action: [ target refactor sourceRegex ];
		parent: #'Code Rewriting';
		order: 600;
		withSeparatorAfter.! !

!NautilusRefactoring class methodsFor: 'menu' stamp: 'BenjaminVanRyseghem 5/13/2012 17:57'!
instVarRefactoringSubmenu: aBuilder
	<nautilusClassMenu>
	| target selectedClass |
	
	target := aBuilder model.
	selectedClass := target selectedClass.
	selectedClass ifNil: [^target].
	
	(aBuilder item: #'Add')
		action: [ target refactor addInstVarFrom: selectedClass ];
		parent: #'Inst Var Refactoring';
		order: 0.
		
	(aBuilder item: #'Rename')
		action: [ target refactor renameInstVarFrom: selectedClass ];
		parent: #'Inst Var Refactoring';
		order: 100.
	
	(aBuilder item: #'Remove')
		action: [ target refactor removeInstVarFrom: selectedClass ];
		parent: #'Inst Var Refactoring';
		icon: (target iconClass iconNamed: #removeIcon);
		order: 200;
		withSeparatorAfter.
		
	(aBuilder item: #'Abstract')
		action: [ target refactor abstractInstVarFrom: selectedClass ];
		parent: #'Inst Var Refactoring';
		order: 300.
	
	(aBuilder item: #'Accessors')
		action: [ target refactor accessorsInstVarFrom: selectedClass ];
		parent: #'Inst Var Refactoring';
		order: 400.
		
	(aBuilder item: #'Pull up')
		action: [ target refactor pullUpInstVarFrom: selectedClass ];
		parent: #'Inst Var Refactoring';
		order: 500.
	
	(aBuilder item: #'Push down')
		action: [ target refactor pushDownInstVarFrom: selectedClass ];
		parent: #'Inst Var Refactoring';
		order: 600.! !

!NautilusRefactoring class methodsFor: 'menu' stamp: 'BenjaminVanRyseghem 5/13/2012 17:59'!
methodRefactoringSubMenu: aBuilder
	<nautilusMethodMenu>
	| target selectedMethod selectedMethods |
	
	target := aBuilder model.
	selectedMethod := target selectedMethod.
	selectedMethods := target selectedMethods.
	selectedMethod ifNil:[ ^ target ].
	
	(aBuilder item: #'Add a parameter')
			action: [ target refactor addAParameterFor: selectedMethod ];
			order: 0;
			parent: #'Refactoring'.
			
	(aBuilder item: #'Inline parameter')
			action: [ target refactor inlineParameterFor: selectedMethod ];
			order: 100;
			parent: #'Refactoring'.
			
	(aBuilder item: #'Inline target sends')
			action: [ target refactor inlineAllSendersFor: selectedMethod ];
			order: 200;
			parent: #'Refactoring'.
			
	(aBuilder item: #'Move')
			action: [ target refactor moveMethodFor: selectedMethod ];
			order: 300;
			parent: #'Refactoring'.
			
	(aBuilder item: #'Swap')
			action: [ target refactor swapMethodFor: selectedMethods ];
			order: 400;
			label: (target selectedClass isMeta
						ifTrue: [ 'Move to instance side' ]
						ifFalse:[ 'Move to class side']);
			parent: #'Refactoring'.
		
	(aBuilder item: #'Push up')
			action: [ target refactor pullUpMethodsFor: selectedMethods ];
			order: 500;
			parent: #'Refactoring'.
			
	(aBuilder item: #'Push down')
			action: [ target refactor pushDownMethodsFor: selectedMethods ];
			order: 600;
			parent: #'Refactoring'.
			
	(aBuilder item: #'Remove')
			action: [ target refactor removeMethodsFor: selectedMethods ];
			order: 700;
			parent: #'Refactoring'.
		
	(aBuilder item: #'Remove parameter')
			action: [ target refactor removeParameterFor: selectedMethod ];
			order: 800;
			enabledBlock: [ selectedMethods size < 2 ];
			parent: #'Refactoring';
			withSeparatorAfter.
				
	(aBuilder item: #'Undo')
			order: 900;
			label: target refactor undoLabel;
			action: [ target refactor undoOperation ];
			enabled: target refactor undoEnabled;
			parent: #'Refactoring'.
	
	(aBuilder item: #'Redo')
			order: 1000;
			label: target refactor redoLabel;
			action: [ target refactor redoOperation ];
			enabled: target refactor redoEnabled;
			parent: #'Refactoring';
			withSeparatorAfter.! !

!NautilusRefactoring class methodsFor: 'instance creation' stamp: ''!
model: model

	^ self new
		model: model;
		yourself! !

!NautilusRefactoring class methodsFor: 'accessing' stamp: ''!
promptOnRefactoring

	^ PromptOnRefactoring ifNil: [ PromptOnRefactoring := true ]! !

!NautilusRefactoring class methodsFor: 'menu' stamp: 'BenjaminVanRyseghem 5/13/2012 17:53'!
refactoringClassMenu: aBuilder
	<nautilusClassMenu>
	| target |
	target := aBuilder model.
	target selectedClass ifNil: [^target].
	
	(aBuilder item: #'Refactoring')
			order: -100.
			
	(aBuilder item: #'Rename inst var')
		action: [ target refactor renameInstVarFrom: target selectedClass ];
		order: -98.
				
	(aBuilder item: #'Rename class var')
		action: [ target refactor renameClassVarFrom: target selectedClass ];
		order: -97.
		
	(aBuilder item: #'Code Rewriting')
			order: 0;
			parent: #'Refactoring'.
			
	(aBuilder item: #'Class Refactoring')
			order: 100;
			parent: #'Refactoring';
			withSeparatorAfter.
	
	(aBuilder item: #'Class Var Refactoring')
			order: 200;
			parent: #'Refactoring'.
			
	(aBuilder item: #'Inst Var Refactoring')
			order: 300;
			parent: #'Refactoring';
			withSeparatorAfter.
			
	(aBuilder item: #'Undo')
			order: 400;
			label: target refactor undoLabel;
			action: [ target refactor undoOperation ];
			enabled: target refactor undoEnabled;
			parent: #'Refactoring'.
	
	(aBuilder item: #'Redo')
			order: 500;
			label: target refactor redoLabel;
			action: [ target refactor redoOperation ];
			enabled: target refactor redoEnabled;
			parent: #'Refactoring';
			withSeparatorAfter.! !

!NautilusRefactoring class methodsFor: 'menu' stamp: 'BenjaminVanRyseghem 5/13/2012 18:00'!
refactoringMethodMenu: aBuilder
	<nautilusMethodMenu>
	| target |
	target := aBuilder model.
	target selectedMethod ifNil:[ ^ target ].
	
	(aBuilder item: #'Refactoring')
			order: -100.
			
	(aBuilder item: #'Rename method (r)')
			action: [ | scroll |
				scroll := target methodWidget vScrollValue.
				target refactor renameMethodFor: target selectedMethod.
				target methodWidget vScrollValue: scroll ];
			order: -90;
			withSeparatorAfter! !

!NautilusRefactoring class methodsFor: 'menu' stamp: ''!
sourceCodeRefactoringMenu: aBuilder
	<nautilusSourceCodeMenu>
	| target |
	target := aBuilder model.
	(aBuilder item: #'Create cascade')
			action: [ 
				target okToChange ifFalse: [ ^ target ].
				target refactor createCascadeBetween: target sourceTextArea selectionInterval from: target selectedMethod ];
			parent: #'Source code refactoring';
			order: 0.
			
	(aBuilder item: #'Extract method')
			action: [ 
				target okToChange ifFalse: [ ^ target ].
				target refactor extractBetween: target sourceTextArea selectionInterval from: target selectedMethod ];
			parent: #'Source code refactoring';
			order: 100.
			
	(aBuilder item: #'Extract method to component')
			action: [ 
				target okToChange ifFalse: [ ^ target ].
				target refactor extractToComponentBetween: target sourceTextArea selectionInterval from: target selectedMethod ];
			parent: #'Source code refactoring';
			order: 200.
		
	(aBuilder item: #'Extract to temporary')
			action: [ 
				target okToChange ifFalse: [ ^ target ].
				target refactor extractToTemporaryBetween: target sourceTextArea selectionInterval from: target selectedMethod ];
			parent: #'Source code refactoring';
			order: 300.
			
	(aBuilder item: #'Inline method')
			action: [ 
				target okToChange ifFalse: [ ^ target ].
				target refactor inlineMethodBetween: target sourceTextArea selectionInterval from: target selectedMethod ];
			parent: #'Source code refactoring';
			order: 400.
			
	(aBuilder item: #'Inline method from component')
			action: [ 
				target okToChange ifFalse: [ ^ target ].
				target refactor inlineMethodFromComponentBetween: target sourceTextArea selectionInterval from: target selectedMethod ];
			parent: #'Source code refactoring';
			order: 500.
			
	(aBuilder item: #'Inline temporary')
			action: [ 
				target okToChange ifFalse: [ ^ target ].
				target refactor inlineTemporaryBetween: target sourceTextArea selectionInterval from: target selectedMethod ];
			parent: #'Source code refactoring';
			order: 600.
			
	(aBuilder item: #'Move variable definition')
			action: [ 
				target okToChange ifFalse: [ ^ target ].
				target refactor moveVariableDefinitionBetween: target sourceTextArea selectionInterval from: target selectedMethod ];
			parent: #'Source code refactoring';
			order: 700.
				
	(aBuilder item: #'Rename temporary')
			action: [ 
				target okToChange ifFalse: [ ^ target ].
				target refactor renameTemporaryNamed: target sourceTextArea selectedContents Between: target sourceTextArea selectionInterval from: target selectedMethod ];
			parent: #'Source code refactoring';
			order: 800.	
						
	(aBuilder item: #'Split cascade')
			action: [ 
				target okToChange ifFalse: [ ^ target ].
				target refactor splitCascadeBetween: target sourceTextArea selectionInterval from: target selectedMethod ];
			parent: #'Source code refactoring';
			order: 900.	
					
	(aBuilder item: #'Temporary to instvar')
			action: [ 
				target okToChange ifFalse: [ ^ target ].
				target refactor temporaryToInstanceVariableNamed: target sourceTextArea selectedContents asString Between: target sourceTextArea selectionInterval from: target selectedMethod ];
			parent: #'Source code refactoring';
			order: 1000;
			withSeparatorAfter.
			
		(aBuilder item: #'Undo')
			order: 1100;
			label: target refactor undoLabel;
			action: [ target refactor undoOperation ];
			enabled: target refactor undoEnabled;
			parent: #'Source code refactoring'.
	
	(aBuilder item: #'Redo')
			order: 1200;
			label: target refactor redoLabel;
			action: [ target refactor redoOperation ];
			enabled: target refactor redoEnabled;
			parent: #'Source code refactoring';
			withSeparatorAfter.! !

!NautilusRefactoring class methodsFor: 'menu' stamp: 'BenjaminVanRyseghem 4/3/2012 13:20'!
sourceCodeRefactoringMenuHolder: aBuilder
	<nautilusSourceCodeMenu>
	| target |
	target := aBuilder model.
	(aBuilder item: #'Source code refactoring')
			order: 9997;
			arguments: {};
			withSeparatorAfter.! !

!NautilusRefactoring methodsFor: 'class var' stamp: ''!
abstractClassVarFrom: aClass
	| refactoring |
	
	aClass ifNil: [ ^ self ].
	refactoring := self privateAbstractClassVarFrom: aClass.
	
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'inst var' stamp: ''!
abstractInstVarFrom: aClass
	| refactoring |
	
	aClass ifNil: [ ^ self ].
	refactoring := self privateAbstractInstVarFrom: aClass.
	
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'class var' stamp: ''!
accessorsClassVarFrom: aClass
	| refactoring |
	
	aClass ifNil: [ ^ self ].
	refactoring := self privateAccessorsClassVarFrom: aClass.
	
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'inst var' stamp: ''!
accessorsInstVarFrom: aClass
	| refactoring |
	
	aClass ifNil: [ ^ self ].
	refactoring := self privateAccessorsInstVarFrom: aClass.
	
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'method' stamp: ''!
addAParameterFor: aMethod
	| refactoring | 
	
	aMethod ifNil: [ ^ self ].
	refactoring := self privateAddAParameterFor: aMethod.
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'class var' stamp: ''!
addClassVarFrom: aClass
	| refactoring |
	
	aClass ifNil: [ ^ self ].
	refactoring := self privateAddClassVarFrom: aClass.
	
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'inst var' stamp: ''!
addInstVarFrom: aClass
	| refactoring |
	
	aClass ifNil: [ ^ self ].
	refactoring := self privateAddInstVarFrom: aClass.
	
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'class' stamp: ''!
buildRenameRefactoringEngineFor: aClass

	| newName |
	newName := (UIManager default
			request: 'Enter the new class name:'
			initialAnswer: aClass name).
	newName isEmptyOrNil ifTrue: [ ^ nil ].
	^ RBRenameClassRefactoring
		model: RBNamespace new
		rename: aClass
		to: newName! !

!NautilusRefactoring methodsFor: 'rewrite code' stamp: ''!
categoryRegex
	
	self model sourceCode: self categoryRegexText! !

!NautilusRefactoring methodsFor: 'rewrite code' stamp: ''!
categoryRegexText

	^ 'RBCategoryRegexRefactoring new
	replace: ''^Kernel-(.*)$'' with: ''System-$1'' ignoreCase: false;
	yourself'! !

!NautilusRefactoring methodsFor: 'undo-redo' stamp: ''!
changeManager
	
	^ RBRefactoryChangeManager instance! !

!NautilusRefactoring methodsFor: 'class' stamp: 'BenjaminVanRyseghem 4/18/2012 17:11'!
changeSuperclassOf: aClass 
	
	"should be implemented if it really makes sense"
	self model sourceTextArea flash! !

!NautilusRefactoring methodsFor: 'display' stamp: ''!
chooseFrom: anArray title: aString lines: aCollection 
	anArray isEmpty
		ifTrue: [ ^ nil ].
	anArray size = 1
		ifTrue: [ ^ anArray first ].
		
	^ UIManager default chooseOrRequestFrom: anArray lines: aCollection title: aString.! !

!NautilusRefactoring methodsFor: 'display' stamp: ''!
chooseMultipleFrom: anArray

	^ self chooseMultipleFrom: anArray title: ''! !

!NautilusRefactoring methodsFor: 'display' stamp: ''!
chooseMultipleFrom: anArray title: aString

	| window |
	window :=  TickDialogWindow 
					itemsList: anArray 
					itemsHeaderName: '' 
					title: aString
					defaultValue: true.

	^ anArray isEmpty
		ifTrue: [ anArray copyEmpty ]
		ifFalse: [ window chooseFromOwner: self model window ]! !

!NautilusRefactoring methodsFor: 'display' stamp: ''!
class: aClass andClassVariable: aBlock
	| variables index variable class |
	
	variables := aClass theNonMetaClass allClassVarNames asArray sort.
	index := UIManager default chooseFrom: variables.
	variable := variables at: index ifAbsent: [ ^ nil ].
	class := aClass theNonMetaClass whichClassDefinesClassVar: variable.
	^ aBlock value: class value: variable! !

!NautilusRefactoring methodsFor: 'display' stamp: ''!
class: aClass andInstVariable: aBlock
	| variables index variable class |
	
	variables := aClass theNonMetaClass allInstVarNames asArray sort.
	index := UIManager default chooseFrom: variables.
	variable := variables at: index ifAbsent: [ ^ nil ].
	class := aClass theNonMetaClass whichClassDefinesInstVar: variable.
	class := class isMeta not
				ifTrue: [ RBClass existingNamed: class name ] 
				ifFalse: [ RBMetaclass existingNamed: class theNonMetaClass name ].
	class model: environment.
	^ aBlock value: class value: variable! !

!NautilusRefactoring methodsFor: 'private-class' stamp: ''!
classObjectFor: anObject 

	(anObject isBehavior or: [anObject isTrait]) ifTrue: [ ^ environment classFor: anObject ].
	anObject isSymbol ifTrue: [ ^ environment classNamed: anObject ].
	^ anObject! !

!NautilusRefactoring methodsFor: 'rewrite code' stamp: ''!
classRegex

	self model sourceCode: self classRegexText! !

!NautilusRefactoring methodsFor: 'rewrite code' stamp: ''!
classRegexText

	^ 'RBClassRegexRefactoring new

	"Example 1: Change class prefixes"
	renameClasses;
	replace: ''^AB(.*)$'' with: ''CD$1'' ignoreCase: false;

	"Example 2: Generate empty test classes"
	createClasses;
	rootClass: TestCase;
	replace: ''^.*$'' with: ''$0Test'' ignoreCase: false;

	"Example 3: Copy classes"
	copyClasses;
	replace: ''^.*$'' with: ''$0Plus'' ignoreCase: false;

	yourself'! !

!NautilusRefactoring methodsFor: 'source' stamp: ''!
createCascadeBetween: anInterval from: aMethod
	| refactoring | 
	
	aMethod ifNil: [ ^ self ].
	refactoring := self privateCreateCascadeBetween: anInterval from: aMethod.
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'source' stamp: ''!
extractBetween: anInterval from: aMethod
	| refactoring | 
	
	aMethod ifNil: [ ^ self ].
	refactoring := self privateExtractBetween: anInterval from: aMethod.
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'source' stamp: ''!
extractToComponentBetween: anInterval from: aMethod
	| refactoring | 
	
	aMethod ifNil: [ ^ self ].
	refactoring := self privateExtractToComponentBetween: anInterval from: aMethod.
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'source' stamp: 'BenjaminVanRyseghem 4/18/2012 17:45'!
extractToMethodTextSelection
	| aMethod selectedInterval textArea |
	
	textArea := self model sourceTextArea.
	aMethod := self model selectedMethod.
	aMethod ifNil: [ ^ textArea flash ].
	textArea selectedContents asString ifEmpty: [ ^ textArea flash ].
	
	selectedInterval := textArea selectionInterval.
	self extractBetween: selectedInterval from: aMethod! !

!NautilusRefactoring methodsFor: 'source' stamp: 'BenjaminVanRyseghem 4/18/2012 17:44'!
extractToTempTextSelection
	| aMethod selectedInterval textArea |
	
	textArea := self model sourceTextArea.
	aMethod := self model selectedMethod.
	aMethod ifNil: [ ^ textArea flash ].
	textArea selectedContents asString ifEmpty: [ ^ textArea flash ].
	
	selectedInterval := textArea selectionInterval.
	self extractToTemporaryBetween: selectedInterval from: aMethod! !

!NautilusRefactoring methodsFor: 'source' stamp: ''!
extractToTemporaryBetween: anInterval from: aMethod
	| refactoring | 
	
	aMethod ifNil: [ ^ self ].
	refactoring := self privateExtractToTemporaryBetween: anInterval from: aMethod.
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'class' stamp: ''!
generateAccessors

	| refactorings |
	refactorings := self privateGenerateAccessorsFor: self model selectedClass.
	refactorings ifNil: [ ^ self ].
	refactorings do: [:each | each model environment: self model browsedEnvironment ].
	self performRefactorings: refactorings! !

!NautilusRefactoring methodsFor: 'class' stamp: ''!
generateSubclass

	| refactoring |
	refactoring := self privateGenerateSubclassFor: self model selectedClass.
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring ! !

!NautilusRefactoring methodsFor: 'class' stamp: ''!
generateSuperClass
	"Execute the refactoring of the receiver."

	| refactoring |
	refactoring := self privateGenerateSuperClassFor: self model selectedClass.
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring.! !

!NautilusRefactoring methodsFor: 'display' stamp: ''!
handleError: anException
	anException actionBlock isNil
		ifTrue: [ self inform: anException messageText ]
		ifFalse: [ 
			(self confirm: anException messageText) 
				ifTrue: [ anException actionBlock value ] ].
	anException return! !

!NautilusRefactoring methodsFor: 'display' stamp: ''!
handleMethodNameRequest: aMethodName

	^ (MethodNameEditor openOn: aMethodName) methodName! !

!NautilusRefactoring methodsFor: 'display' stamp: ''!
handleWarning: anException 
	| message |
	message := (anException messageText endsWith: '?')
		ifTrue: [ anException messageText ]
		ifFalse: [ anException messageText , String cr , 'Do you want to proceed?' ].
	(self confirm: message)
		ifTrue: [ anException resume ]
		ifFalse: [ anException return ]! !

!NautilusRefactoring methodsFor: 'method' stamp: ''!
inlineAllSendersFor: aMethod
	| refactoring | 
	
	aMethod ifNil: [ ^ self ].
	refactoring := self privateInlineAllSendersFor: aMethod.
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'source' stamp: ''!
inlineMethodBetween: anInterval from: aMethod
	| refactoring | 
	
	aMethod ifNil: [ ^ self ].
	refactoring := self privateInlineMethodBetween: anInterval from: aMethod.
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'source' stamp: ''!
inlineMethodFromComponentBetween: anInterval from: aMethod
	| refactoring | 
	
	aMethod ifNil: [ ^ self ].
	refactoring := self privateInlineMethodFromComponentBetween: anInterval from: aMethod.
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'method' stamp: ''!
inlineParameterFor: aMethod
	| refactoring | 
	
	aMethod ifNil: [ ^ self ].
	refactoring := self privateInlineParameterFor: aMethod.
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'source' stamp: ''!
inlineTemporaryBetween: anInterval from: aMethod
	| refactoring | 
	
	aMethod ifNil: [ ^ self ].
	refactoring := self privateInlineTemporaryBetween: anInterval from: aMethod.
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'performing' stamp: ''!
internalPerformRefactorings: aCollectionRefactoring
	"Try to properly label aRefactoring and perform it or open the changes browser, depending on the preferences of the user."

		self promptOnRefactoring
			ifFalse: [ aCollectionRefactoring do: [:e |
												self refactoringOptions: e.
												e execute ]]
			ifTrue: [
				aCollectionRefactoring do: [:e |
												self refactoringOptions: e.
												e primitiveExecute ].
				aCollectionRefactoring ifEmpty: [ ^ self ].
		
				(ChangesBrowser changes: aCollectionRefactoring) openWithSpec ].! !

!NautilusRefactoring methodsFor: 'accessing' stamp: ''!
model
	
	^ model! !

!NautilusRefactoring methodsFor: 'accessing' stamp: ''!
model: aNautilusUI
	
	model := aNautilusUI.
	
	environment := (RBNamespace onEnvironment: aNautilusUI browsedEnvironment)
						name: self printString;
						yourself.! !

!NautilusRefactoring methodsFor: 'method' stamp: ''!
moveMethodFor: aMethod
	| refactoring | 
	
	aMethod ifNil: [ ^ self ].
	refactoring := self privateMoveMethodFor: aMethod.
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'source' stamp: ''!
moveVariableDefinitionBetween: anInterval from: aMethod
	| refactoring | 
	
	aMethod ifNil: [ ^ self ].
	refactoring := self privateMoveVariableDefinitionBetween: anInterval from: aMethod.
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'option' stamp: ''!
openEnvironment: anEnvironment for: aRefactoring
	
	anEnvironment isEmpty
		ifTrue: [ ^ self inform: 'Empty scope' ].
	
	Smalltalk tools browser fullOnEnvironment: anEnvironment! !

!NautilusRefactoring methodsFor: 'performing' stamp: ''!
performRefactoring: aRefactoring
	"Try to properly label aRefactoring and perform it or open the changes browser, depending on the preferences of the user."

	self performRefactorings: {aRefactoring}! !

!NautilusRefactoring methodsFor: 'performing' stamp: ''!
performRefactorings: aCollectionRefactoring

		[[ self internalPerformRefactorings: aCollectionRefactoring ]
			on: RBRefactoringWarning
			do: [ :exception | self handleWarning: exception ]]
			on: RBRefactoringError
			do: [ :exception | self handleError: exception ]! !

!NautilusRefactoring methodsFor: 'private-class var' stamp: ''!
privateAbstractClassVarFrom: aClass

	^ self class: aClass andClassVariable: [ :class :variable | 
		RBAbstractClassVariableRefactoring
			model: environment
			variable: variable
			class: class ]! !

!NautilusRefactoring methodsFor: 'private-inst var' stamp: ''!
privateAbstractInstVarFrom: aClass

	^ self class: aClass andInstVariable: [ :class :variable | 
		RBAbstractInstanceVariableRefactoring
			model: environment
			variable: variable
			class: class ]! !

!NautilusRefactoring methodsFor: 'private-class var' stamp: ''!
privateAccessorsClassVarFrom: aClass

	^ self class: aClass andClassVariable: [ :class :variable | 
		RBCreateAccessorsForVariableRefactoring 
			model: environment
			variable: variable
			class: class
			classVariable: true ]! !

!NautilusRefactoring methodsFor: 'private-inst var' stamp: ''!
privateAccessorsInstVarFrom: aClass

	^ self class: aClass andInstVariable: [ :class :variable | 
		RBCreateAccessorsForVariableRefactoring 
			model: environment
			variable: variable
			class: class
			classVariable: false ]! !

!NautilusRefactoring methodsFor: 'private-method' stamp: 'BenjaminVanRyseghem 3/26/2012 17:46'!
privateAddAParameterFor: aMethod
	| initializer newSelector initialAnswer oldSelector |
	
	oldSelector := aMethod selector.
	
	initialAnswer := oldSelector numArgs = 0
		ifTrue: [ oldSelector , ':' ]
		ifFalse: [ oldSelector ].
		
	newSelector := self request: 'Enter new selector:' initialAnswer: initialAnswer.
	newSelector isEmptyOrNil ifTrue: [ ^ nil ].
	
	initializer := self request: 'Enter default value for parameter:' initialAnswer: 'nil'.
	initializer isEmpty ifTrue: [ ^ nil ].
	
	^ RBAddParameterRefactoring
		model: environment
		addParameterToMethod: aMethod selector
		in: aMethod methodClass
		newSelector: newSelector asSymbol
		initializer: initializer
! !

!NautilusRefactoring methodsFor: 'private-class var' stamp: ''!
privateAddClassVarFrom: aClass
	| name |
	name := (self
				request: 'Enter the new variable name:'
				initialAnswer: 'Var').

	^ RBAddClassVariableRefactoring
		model: environment
		variable: name
		class: aClass theNonMetaClass! !

!NautilusRefactoring methodsFor: 'private-inst var' stamp: ''!
privateAddInstVarFrom: aClass
	| name |
	name := (self
				request: 'Enter the new variable name:'
				initialAnswer: 'inst').

	^ RBAddInstanceVariableRefactoring
		model: environment
		variable: name
		class: aClass theNonMetaClass! !

!NautilusRefactoring methodsFor: 'private-source' stamp: ''!
privateCreateCascadeBetween: aSelection from: aMethod

	^ RBCreateCascadeRefactoring 
		model: environment
		combine: aSelection
		from: aMethod selector
		in: aMethod methodClass! !

!NautilusRefactoring methodsFor: 'private-source' stamp: ''!
privateExtractBetween: anInterval from: aMethod

	^ RBExtractMethodRefactoring
		model: environment
		extract: anInterval
		from: aMethod selector
		in: aMethod methodClass! !

!NautilusRefactoring methodsFor: 'private-source' stamp: ''!
privateExtractToComponentBetween: anInterval from: aMethod

	^ RBExtractMethodToComponentRefactoring
		model: environment
		extract: anInterval
		from: aMethod selector
		in: aMethod methodClass! !

!NautilusRefactoring methodsFor: 'private-source' stamp: ''!
privateExtractToTemporaryBetween: anInterval from: aMethod
	| name |
	
	name := (self request: 'Enter the new variable name:').
	name ifNil: [ ^ nil ].

	^ RBExtractToTemporaryRefactoring
		model: environment
		extract: anInterval
		to: name
		from: aMethod selector
		in: aMethod methodClass! !

!NautilusRefactoring methodsFor: 'private-class' stamp: ''!
privateGenerateAccessorsFor: class

	^ class instVarNames collect: [:aVarName | 
				RBCreateAccessorsForVariableRefactoring
					model: environment
					variable: aVarName
					class: class
					classVariable: false ]! !

!NautilusRefactoring methodsFor: 'private-class' stamp: ''!
privateGenerateSubclassFor: class

	| subclassName |
	subclassName := self request: 'Enter new subclass name:'.
	
	^ RBAddClassRefactoring
		model: environment
		addClass: subclassName
		superclass: class
		subclasses: (self
			chooseMultipleFrom: class subclasses
			title: 'Select subclasses of ' , subclassName , ':')
		category: class category! !

!NautilusRefactoring methodsFor: 'private-class' stamp: ''!
privateGenerateSuperClassFor: class

	| superclassName subclasses |
	superclassName := self request: 'Enter new superclass name:'.
	superclassName isEmptyOrNil ifTrue: [ ^ self ].
	subclasses := self
						chooseMultipleFrom: class subclasses
						title: 'Select subclasses of ' , superclassName , ':'.
						
	subclasses ifNil: [ ^ self ].					
	
	^ RBChildrenToSiblingsRefactoring 
		model: environment
		name: superclassName
		class: class
		subclasses: subclasses! !

!NautilusRefactoring methodsFor: 'private-method' stamp: ''!
privateInlineAllSendersFor: aMethod
	
	^ RBInlineAllSendersRefactoring
		model: environment
		sendersOf: aMethod selector
		in: aMethod methodClass! !

!NautilusRefactoring methodsFor: 'private-source' stamp: ''!
privateInlineMethodBetween: anInterval from: aMethod

	^ RBInlineMethodRefactoring
		model: environment
		inline: anInterval
		inMethod: aMethod selector
		forClass: aMethod methodClass! !

!NautilusRefactoring methodsFor: 'private-source' stamp: ''!
privateInlineMethodFromComponentBetween: anInterval from: aMethod

	^ RBInlineMethodFromComponentRefactoring
		model: environment
		inline: anInterval
		inMethod: aMethod selector
		forClass: aMethod methodClass! !

!NautilusRefactoring methodsFor: 'private-method' stamp: ''!
privateInlineParameterFor: aMethod
	| arguments parameter parser |
	
	parser := RBParser new.
	parser errorBlock: [ :error :position | ^ #() ].
	parser initializeParserWith: aMethod sourceCode.
	arguments :=  parser parseMessagePattern argumentNames.
	arguments ifEmpty: [ ^ nil ].
	
	parameter := (UIManager default chooseFrom: arguments).
	parameter ifNil: [ ^ nil ].
	
	^ RBInlineParameterRefactoring
		model: environment
		inlineParameter: (arguments at: parameter ifAbsent: [ ^ nil ])
		in: aMethod methodClass
		selector: aMethod selector! !

!NautilusRefactoring methodsFor: 'private-source' stamp: ''!
privateInlineTemporaryBetween: anInterval from: aMethod

	^ RBInlineTemporaryRefactoring
		model: environment
		inline: anInterval
		from: aMethod selector
		in: aMethod methodClass! !

!NautilusRefactoring methodsFor: 'private-method' stamp: ''!
privateMoveMethodFor: aMethod
	| instVars instVar |
	instVars := aMethod methodClass instVarNames.
	instVars ifNil: [ ^ nil ].
	instVar := UIManager default chooseFrom: instVars.
	instVar ifNil: [ ^ nil ].
	
	
	^ RBMoveMethodRefactoring
		model: environment
		selector: aMethod selector
		class: aMethod methodClass
		variable: (instVars at: instVar ifAbsent: [ ^ nil ])! !

!NautilusRefactoring methodsFor: 'private-source' stamp: ''!
privateMoveVariableDefinitionBetween: anInterval from: aMethod

	^ RBMoveVariableDefinitionRefactoring
		model: environment
		bindTight: anInterval
		in: aMethod methodClass
		selector: aMethod selector! !

!NautilusRefactoring methodsFor: 'private-class var' stamp: ''!
privatePullUpClassVarFrom: aClass

	^ self class: aClass andClassVariable: [ :class :variable | 
		RBPullUpClassVariableRefactoring 
			model: environment
			variable: variable
			class: class ]! !

!NautilusRefactoring methodsFor: 'private-inst var' stamp: ''!
privatePullUpInstVarFrom: aClass

	^ self class: aClass andInstVariable: [ :class :variable | 
		RBPullUpInstanceVariableRefactoring 
			model: environment
			variable: variable
			class: class ]! !

!NautilusRefactoring methodsFor: 'private-method' stamp: ''!
privatePullUpMethodsFor: aCollection
	| selectors class |
	selectors := aCollection collect: #selector.
	class := aCollection first methodClass.
	(aCollection allSatisfy: [:e | e methodClass == class ]) ifFalse: [ ^ nil ].
	
	^ RBPullUpMethodRefactoring
		model: environment
		pullUp: selectors
		from: class! !

!NautilusRefactoring methodsFor: 'private-class var' stamp: ''!
privatePushDownClassVarFrom: aClass

	^ self class: aClass andClassVariable: [ :class :variable | 
		RBPushDownClassVariableRefactoring
			model: environment
			variable: variable
			class: class ]! !

!NautilusRefactoring methodsFor: 'private-inst var' stamp: ''!
privatePushDownInstVarFrom: aClass

	^ self class: aClass andInstVariable: [ :class :variable | 
		RBPushDownInstanceVariableRefactoring
			model: environment
			variable: variable
			class: class ]! !

!NautilusRefactoring methodsFor: 'private-method' stamp: ''!
privatePushDownMethodsFor: aCollection
	| selectors class |
	selectors := aCollection collect: #selector.
	class := aCollection first methodClass.
	(aCollection allSatisfy: [:e | e methodClass == class ]) ifFalse: [ ^ nil ].
	
	^ RBPushDownMethodRefactoring
		model: environment
		pushDown: selectors
		from: class! !

!NautilusRefactoring methodsFor: 'private-class' stamp: ''!
privateRealizeClassFor: aClass
	
	^ RBRealizeClassRefactoring model: environment className: aClass theNonMetaClass name! !

!NautilusRefactoring methodsFor: 'private-class var' stamp: ''!
privateRemoveClassVarFrom: aClass

	^ self class: aClass andClassVariable: [ :class :variable | 
		RBRemoveClassVariableRefactoring
			model: environment
			variable: variable
			class: class ]! !

!NautilusRefactoring methodsFor: 'private-class' stamp: ''!
privateRemoveClassesFor: aCollection
	
	^ RBRemoveClassRefactoring 
			model: environment 
			classNames: (aCollection collect: [:e | e theNonMetaClass name ])! !

!NautilusRefactoring methodsFor: 'private-inst var' stamp: ''!
privateRemoveInstVarFrom: aClass

	^ self class: aClass andInstVariable: [ :class :variable | 
		RBRemoveInstanceVariableRefactoring
			model: environment
			variable: variable
			class: class ]! !

!NautilusRefactoring methodsFor: 'private-method' stamp: ''!
privateRemoveMethodsFor: aCollection
	| selectors class |
	selectors := aCollection collect: #selector.
	class := aCollection first methodClass.
	(aCollection allSatisfy: [:e | e methodClass == class ]) ifFalse: [ ^ nil ].
	
	^ RBRemoveMethodRefactoring
		model: environment
		removeMethods: selectors
		from: class! !

!NautilusRefactoring methodsFor: 'private-method' stamp: 'BenjaminVanRyseghem 3/26/2012 17:49'!
privateRemoveParameterMethodFor: aMethod
	| arguments parameter parser |
	
	parser := RBParser new.
	parser errorBlock: [ :error :position | ^ #() ].
	parser initializeParserWith: aMethod sourceCode.
	arguments :=  parser parseMessagePattern argumentNames.
	arguments ifEmpty: [ ^ nil ].
	
	parameter := (UIManager default chooseFrom: arguments).
	parameter ifNil: [ ^ nil ].
	
	^ RBRemoveParameterRefactoring
		model: environment
		removeParameter: (arguments at: parameter ifAbsent: [ ^ nil ])
		in: aMethod methodClass
		selector: aMethod selector! !

!NautilusRefactoring methodsFor: 'private-class var' stamp: 'BenjaminVanRyseghem 4/18/2012 16:58'!
privateRenameClassVarFrom: aClass

	^ self class: aClass andClassVariable: [ :class :variable || name |
		name := self 
			request: 'Enter the new variable name:'
			initialAnswer: variable.
			
		name ifNil: [ ^ nil ].	
			
		RBRenameClassVariableRefactoring
			model: environment
			rename: variable
			to: name asSymbol
			in: class ]! !

!NautilusRefactoring methodsFor: 'private-class var' stamp: 'BenjaminVanRyseghem 4/18/2012 16:58'!
privateRenameClassVarNamed: variable from: aClass
	| name |

	name := self 
		request: 'Enter the new variable name:'
		initialAnswer: variable.
	
	name ifNil: [ ^ nil ].
		
	^ RBRenameClassVariableRefactoring
		model: environment
		rename: variable
		to: name asSymbol
		in: aClass! !

!NautilusRefactoring methodsFor: 'private-inst var' stamp: 'BenjaminVanRyseghem 4/18/2012 16:57'!
privateRenameInstVarFrom: aClass

	^ self class: aClass andInstVariable: [ :class :variable || name |
		name := self 
			request: 'Enter the new variable name:'
			initialAnswer: variable.

		name ifNil: [ ^ nil ].

		RBRenameInstanceVariableRefactoring
			model: environment
			rename: variable
			to: name asSymbol
			in: class ]! !

!NautilusRefactoring methodsFor: 'private-inst var' stamp: 'BenjaminVanRyseghem 4/18/2012 16:57'!
privateRenameInstVarNamed: variable from: aClass
	| name |
	
	name := self 
		request: 'Enter the new variable name:'
		initialAnswer: variable.

	name ifNil: [ ^ nil ].

	^ RBRenameInstanceVariableRefactoring
		model: environment
		rename: variable
		to: name asSymbol
		in: aClass! !

!NautilusRefactoring methodsFor: 'private-method' stamp: ''!
privateRenameMethodFor: aMethod
	| class selector oldMethodName newMethodName oldArguments argumentPermutation |
	
	class := aMethod methodClass.
	selector := aMethod selector.
	oldArguments := (RBParser parseMethod: (class methodHeaderFor: selector)) argumentNames.
	oldMethodName := RBMethodName 
		selector: selector
		arguments: oldArguments.
	(newMethodName := self requestMethodNameFor: oldMethodName) ifNil: [ ^ nil ].
	argumentPermutation := newMethodName arguments collect: [ :each | oldArguments indexOf: each ].
	
	^ RBRenameMethodRefactoring
		model: environment
		renameMethod: selector
		in: class
		to: newMethodName selector
		permutation: argumentPermutation! !

!NautilusRefactoring methodsFor: 'private-source' stamp: ''!
privateRenameTemporaryNamed: oldname Between: anInterval from: aMethod
	| name |
	name := (self
				request: 'Enter the new variable name:' 
				initialAnswer: oldname).
	name ifNil: [ ^ nil ].

	^ RBRenameTemporaryRefactoring
		model: environment
		renameTemporaryFrom: anInterval
		to: name
		in: aMethod methodClass
		selector: aMethod selector
! !

!NautilusRefactoring methodsFor: 'private-source' stamp: ''!
privateSplitCascadeBetween: anInterval from: aMethod

	^ RBSplitCascadeRefactoring
		model: environment
		split: anInterval
		from: aMethod selector
		in: aMethod methodClass! !

!NautilusRefactoring methodsFor: 'private-class' stamp: ''!
privateSplitClassFor: aClass
	| class |

	class := aClass theNonMetaClass.
	^ RBSplitClassRefactoring
		model: environment
		class: class
		instanceVariables: (self 
			chooseMultipleFrom: class instVarNames
			title: 'Select variables to extract:')
		newClassName: (self request: 'Enter new class name:')
		referenceVariableName: (self request: 'Enter new variable name where requests will be forwarded:')! !

!NautilusRefactoring methodsFor: 'private-method' stamp: ''!
privateSwapMethodFor: aCollection
	
	^ aCollection collect: [:aMethod | 
			RBSwapMethodRefactoring
				model: environment
				swapMethod: aMethod selector
				in: aMethod methodClass ]! !

!NautilusRefactoring methodsFor: 'private-source' stamp: ''!
privateTemporaryToInstanceVariableNamed: name Between: anInterval from: aMethod

	^ RBTemporaryToInstanceVariableRefactoring
		model: environment
		class: aMethod methodClass
		selector: aMethod selector
		variable: name! !

!NautilusRefactoring methodsFor: 'option' stamp: ''!
promptOnRefactoring

	^ self class promptOnRefactoring! !

!NautilusRefactoring methodsFor: 'rewrite code' stamp: ''!
protocolRegex

	self model sourceCode: self protocolRegexText! !

!NautilusRefactoring methodsFor: 'rewrite code' stamp: ''!
protocolRegexText

	^ 'RBProtocolRegexRefactoring new
	replace: ''^\*system(.*)$'' with: ''*kernel$1'' ignoreCase: true;
	yourself'! !

!NautilusRefactoring methodsFor: 'class var' stamp: ''!
pullUpClassVarFrom: aClass
	| refactoring |
	
	aClass ifNil: [ ^ self ].
	refactoring := self privatePullUpClassVarFrom: aClass.
	
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'inst var' stamp: ''!
pullUpInstVarFrom: aClass
	| refactoring |
	
	aClass ifNil: [ ^ self ].
	refactoring := self privatePullUpInstVarFrom: aClass.
	
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'method' stamp: ''!
pullUpMethodsFor: aCollection
	| refactoring | 
	
	aCollection ifEmpty: [ ^ self ].
	refactoring := self privatePullUpMethodsFor: aCollection.
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'class var' stamp: ''!
pushDownClassVarFrom: aClass
	| refactoring |
	
	aClass ifNil: [ ^ self ].
	refactoring := self privatePushDownClassVarFrom: aClass.
	
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'inst var' stamp: ''!
pushDownInstVarFrom: aClass
	| refactoring |
	
	aClass ifNil: [ ^ self ].
	refactoring := self privatePushDownInstVarFrom: aClass.
	
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'method' stamp: ''!
pushDownMethodsFor: aCollection
	| refactoring | 
	
	aCollection ifEmpty: [ ^ self ].
	refactoring := self privatePushDownMethodsFor: aCollection.
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'class' stamp: ''!
realizeClass

	| refactoring |
	refactoring := self privateRealizeClassFor: self model selectedClass.
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring ! !

!NautilusRefactoring methodsFor: 'undo-redo' stamp: ''!
redoEnabled

	^ self changeManager hasRedoableOperations! !

!NautilusRefactoring methodsFor: 'undo-redo' stamp: ''!
redoLabel
	| stream |
	stream := WriteStream with: 'Redo'.
	self changeManager hasRedoableOperations ifFalse: [ ^ stream contents ].
	
	stream nextPut: $ ; nextPutAll: self changeManager redoChange name.
	stream position > 40
		ifTrue: [ stream position: 40; nextPutAll: '...' ].
		
	^ stream contents! !

!NautilusRefactoring methodsFor: 'undo-redo' stamp: ''!
redoOperation 

	self changeManager redoOperation! !

!NautilusRefactoring methodsFor: 'option' stamp: ''!
refactoringOptions: aRefactoring
	aRefactoring
		setOption: #implementorToInline
		toUse: [ :ref :imps | self requestImplementorToInline: imps ];
		setOption: #methodName
		toUse: [ :ref :name | self requestMethodNameFor: name ];
		setOption: #selfArgumentName
		toUse: [ :ref | self requestSelfArgumentName ];
		setOption: #selectVariableToMoveTo
		toUse: [ :ref :class :selector | self selectVariableToMoveMethodTo: selector class: class ];
		setOption: #variableTypes
		toUse: [ :ref :types :selected | self selectVariableTypesFrom: types selected: selected for: ref ];
		setOption: #extractAssignment
		toUse: [ :ref :string | self shouldExtractAssignmentTo: string ];
		setOption: #inlineExpression
		toUse: [ :ref :string | self shouldInlineExpression: string ];
		setOption: #alreadyDefined
		toUse: [ :ref :class :selector | self shouldOverride: selector in: class ];
		setOption: #useExistingMethod
		toUse: [ :ref :selector | self shouldUseExistingMethod: selector ];
		setOption: #openBrowser
		toUse: [ :ref :env | self openEnvironment: env  for: ref]! !

!NautilusRefactoring methodsFor: 'class var' stamp: ''!
removeClassVarFrom: aClass
	| refactoring |
	
	aClass ifNil: [ ^ self ].
	refactoring := self privateRemoveClassVarFrom: aClass.
	
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'class' stamp: ''!
removeClasses: aCollection
	"Execute the refactoring of the receiver."

	| refactoring |
	refactoring := self privateRemoveClassesFor: aCollection.
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.

	self performRefactoring: refactoring.
	^ true! !

!NautilusRefactoring methodsFor: 'inst var' stamp: ''!
removeInstVarFrom: aClass
	| refactoring |
	
	aClass ifNil: [ ^ self ].
	refactoring := self privateRemoveInstVarFrom: aClass.
	
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'method' stamp: ''!
removeMethodsFor: aCollection
	| refactoring | 
	
	aCollection ifEmpty: [ ^ self ].
	refactoring := self privateRemoveMethodsFor: aCollection.
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'method' stamp: ''!
removeParameterFor: aMethod
	| refactoring | 
	
	aMethod ifNil: [ ^ self ].
	refactoring := self privateRemoveParameterMethodFor: aMethod.
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'class' stamp: ''!
renameClass: aClass
	| refactoring |
	
	aClass ifNil: [ ^ self ].
	refactoring := self buildRenameRefactoringEngineFor: aClass.
	
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring ! !

!NautilusRefactoring methodsFor: 'class var' stamp: ''!
renameClassVarFrom: aClass
	| refactoring |
	
	aClass ifNil: [ ^ self ].
	refactoring := self privateRenameClassVarFrom: aClass.

	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'class var' stamp: 'BenjaminVanRyseghem 4/18/2012 16:57'!
renameClassVarNamed: aName from: aClass
	| refactoring |
	
	aClass ifNil: [ ^ self ].
	refactoring := self privateRenameClassVarNamed: aName from: aClass.

	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'inst var' stamp: ''!
renameInstVarFrom: aClass
	| class refactoring |
	
	aClass ifNil: [ ^ self ].
	refactoring := self privateRenameInstVarFrom: aClass.

	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'inst var' stamp: 'BenjaminVanRyseghem 4/18/2012 16:55'!
renameInstVarNamed: aName from: aClass
	| refactoring |
	
	aClass ifNil: [ ^ self ].
	refactoring := self privateRenameInstVarNamed: aName from:  aClass.
	
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'method' stamp: ''!
renameMethodFor: aMethod
	| refactoring | 
	
	aMethod ifNil: [ ^ self ].
	refactoring := self privateRenameMethodFor: aMethod.
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'source' stamp: ''!
renameTemporaryNamed: oldname Between: anInterval from: aMethod
	| refactoring | 
	
	aMethod ifNil: [ ^ self ].
	refactoring := self privateRenameTemporaryNamed: oldname Between: anInterval from: aMethod.
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'source' stamp: 'BenjaminVanRyseghem 4/18/2012 16:47'!
renameTextSelection

	self model selectedMethod 
		ifNotNil: [ ^ self renameTextSelectionForMethod ].
	
	(self model  selectedCategory isNil and: [ self model selectedClass notNil ])	
		ifTrue: [ " we are editing a class definition " 
			^ self renameTextSelectionForClass ].! !

!NautilusRefactoring methodsFor: 'source' stamp: 'BenjaminVanRyseghem 4/18/2012 17:06'!
renameTextSelectionForClass
	| selectedInterval selection class |
	
	class := self model selectedClass.
	selection := self model sourceTextArea selectedContents asString.
	selectedInterval := self model sourceTextArea selectionInterval.
	
	selectedInterval isEmpty 				ifTrue: [ ^ self model sourceTextArea flash ].
	(selection includes: Character space) 	ifTrue: [ ^ self model sourceTextArea flash ].
		
	"Try to rename the superclass"
	selectedInterval first = 1
		ifTrue: [ self changeSuperclassOf: class ].
		
	"Rename the selected class"
	selection = class name
		ifTrue: [ self renameClass: class ].
		
	"Rename a variable"
	self renameVariable: selection.! !

!NautilusRefactoring methodsFor: 'source' stamp: 'BenjaminVanRyseghem 4/18/2012 17:25'!
renameTextSelectionForMethod
	| selectedInterval selection method |
	
	method := self model selectedMethod.
	selection := self model sourceTextArea selectedContents asString.
	selectedInterval := self model sourceTextArea selectionInterval.
	
	selectedInterval first = 1 
		ifTrue: [ ^ self renameMethodFor: method ].
		
	self renameTemporaryNamed: selection Between: selectedInterval from: method! !

!NautilusRefactoring methodsFor: 'source' stamp: 'BenjaminVanRyseghem 4/18/2012 16:57'!
renameVariable: aString 
	| class |
	class := self model selectedClass.
	
	(class instVarNames includes: aString)
		ifTrue: [ ^ self renameInstVarNamed: aString from: class].
		
	(class classVarNames includes: aString)
		ifTrue: [ ^ self renameClassVarNamed: aString from: class].
		
	self model sourceTextArea flash! !

!NautilusRefactoring methodsFor: 'display' stamp: ''!
request: aString

	^ UIManager default request: aString initialAnswer: ''! !

!NautilusRefactoring methodsFor: 'display' stamp: ''!
request: aString initialAnswer: aTemplateString

	^ UIManager default request: aString initialAnswer: aTemplateString
! !

!NautilusRefactoring methodsFor: 'display' stamp: ''!
requestMethodNameFor: aMethodName

	^ self handleMethodNameRequest: aMethodName! !

!NautilusRefactoring methodsFor: 'option' stamp: ''!
requestSelfArgumentName

	^ self request: 'Enter name for argument to refer to "self" in extracted method'! !

!NautilusRefactoring methodsFor: 'rewrite code' stamp: ''!
rewriteCode
	
	self model sourceCode: self rewriteCodeText! !

!NautilusRefactoring methodsFor: 'rewrite code' stamp: ''!
rewriteCodeText
	
	^ 'RBParseTreeRewriter new
	replace: ''`@object'' with: ''`@object'';
	replace: ''`@object'' with: ''`@object'' when: [ :node | true ];
	replace: ''`@object'' withValueFrom: [ :node | node ];
	replace: ''`@object'' withValueFrom: [ :node | node ] when: [ :node | true ];
	replaceMethod: ''`@method: `@args | `@temps | `@.statements'' with: ''`@method: `@args | `@temps | `@.statements'';
	replaceMethod: ''`@method: `@args | `@temps | `@.statements'' with: ''`@method: `@args | `@temps | `@.statements'' when: [ :node | true ];
	replaceMethod: ''`@method: `@args | `@temps | `@.statements'' withValueFrom: [ :node | node ];
	replaceMethod: ''`@method: `@args | `@temps | `@.statements'' withValueFrom: [ :node | node ] when: [ :node | true ];
	yourself'! !

!NautilusRefactoring methodsFor: 'rewrite code' stamp: ''!
searchCode
	
	self model sourceCode: self searchCodeText! !

!NautilusRefactoring methodsFor: 'rewrite code' stamp: ''!
searchCodeText

	^ 'RBParseTreeSearcher new
	matches: ''`@object'' do: [ :node :answer | node ];
	matchesMethod: ''`@method: `@args | `@temps | `@.statements'' do: [ :node :answer | node ];
	yourself'! !

!NautilusRefactoring methodsFor: 'option' stamp: ''!
selectVariableToMoveMethodTo: aSelector class: aClass
	| parseTree nameList | 
	parseTree := aClass parseTreeFor: aSelector.
	parseTree isNil ifTrue: [ 
		parseTree := RBMethodNode 
			selector: #value
			body: (RBSequenceNode statements: #()) ].
	nameList := OrderedCollection new.
	nameList
		addAll: parseTree argumentNames asSortedCollection;
		addAll: aClass allInstanceVariableNames asSortedCollection.
	^ self 
		chooseFrom: nameList 
		title: 'Select variable to move method into:'
		lines: (Array with: parseTree argumentNames size)! !

!NautilusRefactoring methodsFor: 'option' stamp: ''!
selectVariableTypesFrom: aCollectionOfTypes selected: aSelectedCollection for: aRefactoring
	| stream result |
	stream := WriteStream on: String new.
	aCollectionOfTypes
		do: [ :each | stream nextPutAll: each name ]
		separatedBy: [ stream cr ].
	result := UITheme current 
				textEditorIn: self model window
				text: 'Select classes to move to: (one class per line)'
				title: 'Select classes to move to:'
				entryText: stream contents
				entryHeight: 300.
	result isNil
		ifTrue: [ ^ nil ].
	^ (result asString findTokens: String crlf)
		collect: [ :each | aRefactoring model classFor: (Smalltalk classNamed: each trimBoth) ]
		! !

!NautilusRefactoring methodsFor: 'option' stamp: ''!
shouldOverride: aSelector in: aClass 

	^ self confirm: ('<1s> is already defined in the <2p> hierarchy. Extracting it to an existing selector may change behavior. Do you wish to use <1s> anyway?' expandMacrosWith: aSelector with: aClass)! !

!NautilusRefactoring methodsFor: 'display' stamp: 'StephaneDucasse 5/18/2012 14:54'!
shouldUseExistingMethod: aMethodName
	"for now we always want to use the existing method."
	^ true! !

!NautilusRefactoring methodsFor: 'rewrite code' stamp: ''!
sourceRegex
	
	self model sourceCode: self sourceRegexText! !

!NautilusRefactoring methodsFor: 'rewrite code' stamp: ''!
sourceRegexText

	^ 'RBSourceRegexRefactoring new
	
	"Example 1: Replace symbols with strings"
	replace: ''#(\w+)'' with: ''''''$1'''''' ignoreCase: false;
	
	"Example 2: Replace 4 spaces with tabs"
	replace: ''    '' with: ''	'' ignoreCase: false;

	yourself'! !

!NautilusRefactoring methodsFor: 'source' stamp: ''!
splitCascadeBetween: anInterval from: aMethod
	| refactoring | 
	
	aMethod ifNil: [ ^ self ].
	refactoring := self privateSplitCascadeBetween: anInterval from: aMethod.
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'class' stamp: ''!
splitClass: aClass
	| refactoring |
	
	aClass ifNil: [ ^ self ].
	refactoring := self privateSplitClassFor: aClass.
	
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'method' stamp: ''!
swapMethodFor: aCollection
	| refactorings | 
	
	aCollection ifEmpty: [ ^ self ].
	refactorings := self privateSwapMethodFor: aCollection.
	refactorings isEmptyOrNil ifTrue: [ ^ self ].
	refactorings do: [:e | e model environment: self model browsedEnvironment ].
	self performRefactorings: refactorings! !

!NautilusRefactoring methodsFor: 'source' stamp: ''!
temporaryToInstanceVariableNamed: name Between: anInterval from: aMethod
	| refactoring | 
	
	aMethod ifNil: [ ^ self ].
	refactoring := self privateTemporaryToInstanceVariableNamed: name Between: anInterval from: aMethod.
	refactoring ifNil: [ ^ self ].
	refactoring model environment: self model browsedEnvironment.
	self performRefactoring: refactoring! !

!NautilusRefactoring methodsFor: 'rewrite code' stamp: ''!
typeClass
	
	self model sourceCode: (self typeClassTextFor: self selectedClasses)! !

!NautilusRefactoring methodsFor: 'rewrite code' stamp: ''!
typeClassTextFor: aCollectionOfClass

	^ String streamContents: [:s |
		aCollectionOfClass do: [:class |
			s << (RBRefactoryTyper new runOn: class) printString.
			s cr; cr ]]! !

!NautilusRefactoring methodsFor: 'undo-redo' stamp: ''!
undoEnabled

	^ self changeManager hasUndoableOperations! !

!NautilusRefactoring methodsFor: 'undo-redo' stamp: ''!
undoLabel
	| stream |
	stream := WriteStream with: 'Undo'.
	self changeManager hasUndoableOperations ifFalse: [ ^ stream contents ].
	
	stream nextPut: $ ; nextPutAll: self changeManager undoChange name.
	stream position > 40
		ifTrue: [ stream position: 40; nextPutAll: '...' ].
		
	^ stream contents! !

!NautilusRefactoring methodsFor: 'undo-redo' stamp: ''!
undoOperation 

	self changeManager undoOperation! !

ComposableModel subclass: #ChangesBrowser
	instanceVariableNames: 'acceptButton cancelButton pickList textArea'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NautilusRefactoring-Utilities'!

!ChangesBrowser commentStamp: '<historical>' prior: 0!
A ChangesBrowser is a browser used to browse a composite change wich gather all the wanted changes!

!ChangesBrowser class methodsFor: 'instance creation' stamp: ''!
changes: aCollection

	^ self new
		changes: aCollection;
		yourself! !

!ChangesBrowser class methodsFor: 'specs' stamp: 'bvr 6/4/2012 14:39'!
defaultSpec
	<spec>
	| height |
	height := 25.
	^ { #Panel.
			#changeProportionalLayout.
			#addMorphWrapper:.	{	#MorphWrapper.
											#morph:. 	{#model. #pickList.}.
											#fullFrame:. 	{#LayoutFrame.
																#fractions:offsets:.	 ( 0@0 corner: 1@0.5 ).	(0@0 corner: 0@height negated). }}.
			#addMorphWrapper:.	{	#MorphWrapper.
											#morph:. 	{#model. #textArea.}.
											#fullFrame:. 	{#LayoutFrame.
																#fractions:offsets:.	( 0@0.5 corner: 1@1 ).	(0@height negated corner: 0@(height negated)). }}.
			#addMorphWrapper:.	{	#MorphWrapper.
											#morph:. 		{#model. #acceptButton.}.
											#fullFrame:. 	{#LayoutFrame.
																#fractions:offsets:.	 (0@1 corner: 0.5@1).	(0@(height negated) corner: 0@0). }}.										
			#addMorphWrapper:.	{	#MorphWrapper.
											#morph:. 	{#model. #cancelButton.}.
											#fullFrame:. 	{#LayoutFrame.
																#fractions:offsets:.	 (0.5@1 corner: 1@1).	(0@(height negated) corner: 0@0). }}.
			#addPaneSplitters.
			#vResizing:. #spaceFill.
			#hResizing:. #spaceFill}.! !

!ChangesBrowser class methodsFor: 'specs' stamp: ''!
title

	^ 'Changes Browser'! !

!ChangesBrowser methodsFor: 'private' stamp: ''!
accept

	self okToChange ifFalse: [ ^self ].
	self pickedChanges do: [:change | RBRefactoryChangeManager instance performChange: change ].
	window delete! !

!ChangesBrowser methodsFor: 'accessing' stamp: ''!
acceptButton
	^ acceptButton! !

!ChangesBrowser methodsFor: 'private' stamp: ''!
buildDiffFor: aChange

	^ aChange
		ifNil: [ '' ]
		ifNotNil: [
			TextDiffBuilder 
				buildDisplayPatchFrom: aChange oldVersionTextToDisplay 
				to: aChange textToDisplay ].! !

!ChangesBrowser methodsFor: 'private' stamp: ''!
cancel

	window delete! !

!ChangesBrowser methodsFor: 'accessing' stamp: ''!
cancelButton
	^ cancelButton! !

!ChangesBrowser methodsFor: 'private' stamp: ''!
change: aCompositeChange

	pickList items: (aCompositeChange whatToDisplayIn: self)! !

!ChangesBrowser methodsFor: 'protocol' stamp: ''!
changes: aCollection

	| composite |
	composite := RBCompositeRefactoryChange new.
	aCollection do: [:change | composite addChange: change ].
	self change: composite! !

!ChangesBrowser methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 6/11/2012 10:54'!
initialize
	"Initialization code for ChangesBrowser"

	super initialize.
	self bindings: MorphicBindingsWithSplitters new.! !

!ChangesBrowser methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 6/11/2012 10:51'!
initializeAcceptButton

	acceptButton
		state: true;
		enabled: true;
		action: [ self accept ];
		label: 'Accept'! !

!ChangesBrowser methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 6/11/2012 10:52'!
initializeCancelButton

	cancelButton
		state: true;
		enabled: true;
		action: [ self cancel ];
		label: 'Cancel'! !

!ChangesBrowser methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 6/12/2012 17:49'!
initializePickList
		
	pickList
		labelClickable: false;
		defaultValue: true;
		displayBlock: [ :item | item nameToDisplay ]! !

!ChangesBrowser methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 6/12/2012 17:49'!
initializePresenter

	pickList 
		whenSelectedItemChanged: [ :item | textArea text: (self buildDiffFor: item) ]! !

!ChangesBrowser methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 6/12/2012 17:48'!
initializeWidgets

	self instantiateModels:  #(
		cancelButton 	ButtonModel
		acceptButton 	ButtonModel
		pickList 		PickListModel
		textArea 		TextModel ).

	self initializeAcceptButton.
	self initializeCancelButton.
	self initializePickList.
	self setFocus.! !

!ChangesBrowser methodsFor: 'accessing' stamp: ''!
pickList
	^ pickList! !

!ChangesBrowser methodsFor: 'protocol' stamp: ''!
pickedChanges

	^ pickList pickedItems! !

!ChangesBrowser methodsFor: 'initialization' stamp: ''!
setFocus

	self focusOrder
		add: acceptButton;
		add: cancelButton;
		add: pickList;
		add: textArea! !

!ChangesBrowser methodsFor: 'accessing' stamp: ''!
textArea
	^ textArea! !

ComposableModel subclass: #CodeCriticBrowser
	instanceVariableNames: 'environment listModel1 listModel2 listModel3 process status textModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NautilusRefactoring-Utilities'!

!CodeCriticBrowser commentStamp: '<historical>' prior: 0!
A CodeCriticBrowser is a UI made to browse code critics refactor!

!CodeCriticBrowser class methodsFor: 'specs' stamp: 'bvr 6/4/2012 14:41'!
defaultSpec
	<spec>
	^ { #PanelMorph.
			#changeTableLayout.
			#listDirection:.		#bottomToTop.
			#addMorph:. 		self topSpec.
			#hResizing:. 		#spaceFill.
			#vResizing:. 		#spaceFill.
			#addMorph:. 		{	#model.	#textModel. }}! !

!CodeCriticBrowser class methodsFor: 'specs' stamp: ''!
title

	^ 'Code critics'! !

!CodeCriticBrowser class methodsFor: 'specs' stamp: 'bvr 6/4/2012 14:42'!
topSpec

	^ { #PanelMorph.
			#changeTableLayout.
			#listDirection:.		#rightToLeft.
			#hResizing:. 		#spaceFill.
			#vResizing:. 		#spaceFill.
			#addMorph:. 		{	#model.	#listModel1.	}.
			#addMorph:. 		{	#model.	#listModel2.	}.
			#addMorph:. 		{	#model.	#listModel3.	}}! !

!CodeCriticBrowser methodsFor: 'processing' stamp: ''!
basicSearch: aRule
	| numberSelectors precentIncrement percent checker |
	aRule resetResult.
	self status: 'Searching'; refresh.
	numberSelectors := self environment numberSelectors.
	precentIncrement := numberSelectors isZero
		ifFalse: [ 100.0 / numberSelectors ]
		ifTrue: [ 100.0 ].
	percent := 0.0.
	checker := (Smalltalk at: #SmalllintChecker) new.
	checker
		rule: aRule;
		context: (Smalltalk at: #SmalllintChecker) new;
		environment: self environment;
		methodBlock: [ 
			percent := percent + precentIncrement min: 100.
			self status: percent truncated asString , '%' ].
	[ checker run ] ensure: [
		self refresh.
		self status: (self root problemCount > 0
			ifTrue: [ self root problemCount asString , ' problems' ]).
		checker release.
		process := nil ]! !

!CodeCriticBrowser methodsFor: 'accessing' stamp: ''!
environment
	
	^ environment! !

!CodeCriticBrowser methodsFor: 'accessing' stamp: ''!
environment: anObject
	
	environment := anObject! !

!CodeCriticBrowser methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 6/12/2012 17:54'!
initializePresenter
	
	listModel1 whenSelectedItemChanged: [:selection |
		listModel2 resetSelection.
		listModel2 items: (selection 
								ifNil: [ {} ]
								ifNotNil: [ selection rules ])].
	
	listModel2 whenSelectedItemChanged: [:selection |
		listModel3 resetSelection.
		listModel3 items: (selection 
								ifNil: [ {} ]
								ifNotNil: [ selection rules ])].
	
	listModel3 whenSelectedItemChanged: [:selection |
		selection 
			ifNil: [ textModel text: '' ]
			ifNotNil: [ textModel text: selection result label ]]! !

!CodeCriticBrowser methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 6/12/2012 17:54'!
initializeWidgets

	self instantiateModels: #(
		listModel1 	ListComposableModel
		listModel2 	ListComposableModel
		listModel3 	ListComposableModel
		textModel 	TextModel ).
	
	listModel1 displayBlock: [:rule | rule name ].
	listModel2 displayBlock: [:rule | rule name ].
	listModel3 displayBlock: [:rule | rule name ].! !

!CodeCriticBrowser methodsFor: 'accessing' stamp: ''!
listModel1
	
	^ listModel1! !

!CodeCriticBrowser methodsFor: 'accessing' stamp: ''!
listModel1: anObject
	
	listModel1 := anObject! !

!CodeCriticBrowser methodsFor: 'accessing' stamp: ''!
listModel2
	
	^ listModel2! !

!CodeCriticBrowser methodsFor: 'accessing' stamp: ''!
listModel2: anObject

	listModel2 := anObject! !

!CodeCriticBrowser methodsFor: 'accessing' stamp: ''!
listModel3
	
	^ listModel3! !

!CodeCriticBrowser methodsFor: 'accessing' stamp: ''!
listModel3: anObject
	
	listModel3 := anObject! !

!CodeCriticBrowser methodsFor: 'accessing' stamp: ''!
process
	
	^ process! !

!CodeCriticBrowser methodsFor: 'accessing' stamp: ''!
process: anObject
	
	process := anObject! !

!CodeCriticBrowser methodsFor: 'protocol' stamp: ''!
rules: aCollection

	listModel1 items: aCollection! !

!CodeCriticBrowser methodsFor: 'accessing' stamp: ''!
status
	
	^ status! !

!CodeCriticBrowser methodsFor: 'accessing' stamp: ''!
status: anObject
	
	status := anObject! !

!CodeCriticBrowser methodsFor: 'accessing' stamp: ''!
textModel
	
	^ textModel! !

!CodeCriticBrowser methodsFor: 'accessing' stamp: ''!
textModel: anObject
	
	textModel := anObject! !
